#=====================================================================
# SQL-Ledger Accounting
# Copyright (C) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
# chart of accounts
#
# CHANGE LOG:
#   DS. 2000-07-04  Created
#
#======================================================================


package CA;


sub all_accounts {
  my ($self, $myconfig, $form) = @_;

  my $amount = ();
  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $query = qq|SELECT accno,
                 SUM(acc_trans.amount) AS amount
                 FROM chart, acc_trans
		 WHERE chart.id = acc_trans.chart_id
		 GROUP BY accno|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my $ref = $sth->fetchrow_hashref) {
    $amount{$ref->{accno}} = $ref->{amount}
  }
  $sth->finish;
    
  $query = qq|SELECT id, accno, description, charttype, gifi,
              category, link
              FROM chart
	      ORDER BY accno|;
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);
 
  while (my $ca = $sth->fetchrow_hashref) {
    $ca->{amount} = $amount{$ca->{accno}};
    if ($ca->{amount} < 0) {
      $ca->{debit} = $ca->{amount} * -1;
    } else {
      $ca->{credit} = $ca->{amount};
    }
    push @{ $form->{CA} }, $ca;
  }

  $sth->finish;
  $dbh->disconnect;

}


sub all_transactions {
  my ($self, $myconfig, $form) = @_;

  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  # get chart_id
  my $query = qq|SELECT id FROM chart
                 WHERE accno = $form->{accno}|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  my ($id) = $sth->fetchrow_array;
  $sth->finish;

  my $where = 'true';
  # build WHERE clause from dates if any
  if ($form->{fromdate}) {
    $where .= " AND acc_trans.transdate >= '$form->{fromdate}'";
  }
  if ($form->{todate}) {
    $where .= " AND acc_trans.transdate <= '$form->{todate}'";
  }

  # get all transactions
  $query = 
    qq|SELECT gl.id, gl.source, gl.description,
              acc_trans.transdate, FALSE AS invoice,
	      (CASE WHEN acc_trans.amount < 0 THEN -acc_trans.amount END)
	        AS debit,
	      (CASE WHEN acc_trans.amount > 0 THEN acc_trans.amount END)
	        AS credit,
	      'gl' as charttype
              FROM gl, acc_trans
              WHERE $where
              AND acc_trans.chart_id = $id
              AND acc_trans.trans_id = gl.id
    UNION
    SELECT ar.id, ar.invnumber, customer.name,
              acc_trans.transdate, ar.invoice,
	      (CASE WHEN acc_trans.amount < 0 THEN -acc_trans.amount END)
	        AS debit,
	      (CASE WHEN acc_trans.amount > 0 THEN acc_trans.amount END)
	        AS credit,
	      'ar' as charttype
              FROM ar, acc_trans
              WHERE $where
              AND acc_trans.chart_id = $id
              AND acc_trans.trans_id = ar.id
              AND ar.customer = customer.id
    UNION
    SELECT ap.id, ap.invnumber, vendor.name,
              acc_trans.transdate, ap.invoice,
	      (CASE WHEN acc_trans.amount < 0 THEN -acc_trans.amount END)
	        AS debit,
	      (CASE WHEN acc_trans.amount > 0 THEN acc_trans.amount END)
	        AS credit,
	      'ap' as charttype
              FROM ap, acc_trans
              WHERE $where
              AND acc_trans.chart_id = $id
              AND acc_trans.trans_id = ap.id
              AND ap.vendor = vendor.id
    ORDER BY $form->{sort}|;

  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my $ca = $sth->fetchrow_hashref) {

    # gl
    if ($ca->{charttype} eq "gl") {
      $ca->{module} = "gl";
    }

    # ap
    if ($ca->{charttype} eq "ap") {
      if ($ca->{invoice}) {
	$ca->{module} = "ir";
      } else {
	$ca->{module} = "ap";
      }
    }

    # ar
    if ($ca->{charttype} eq "ar") {
      if ($ca->{invoice}) {
	$ca->{module} = "is";
      } else {
	$ca->{module} = "ar";
      }
    }

    push @{ $form->{CA} }, $ca;

  }
 
  $sth->finish;
  $dbh->disconnect;

}

1;

