#=====================================================================
# SQL-Ledger Accounting
# Copyright (C) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
#
# Inventory Control backend
#
# CHANGE LOG:
#   DS. 2000-07-04  Created
#   DS. 2001-06-13  Changed account number links from accno to id
#   DS. 2001-06-30  Added assemblies
#
#======================================================================

package IC;


sub get_part {
  my ($self, $myconfig, $form) = @_;

  # connect to db
  my $dbh = $form->dbconnect($myconfig);

  my $query = qq|SELECT *,
                      (SELECT accno FROM chart
		      WHERE inventory_accno_id = chart.id) AS inventory_accno,
                      (SELECT accno FROM chart
		      WHERE income_accno_id = chart.id) AS income_accno,
                      (SELECT accno FROM chart
		      WHERE expense_accno_id = chart.id) AS expense_accno
	         FROM parts
                 WHERE id = $form->{id}|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);
  my $ref = $sth->fetchrow_hashref;

  # copy to $form variables
  foreach my $key ( keys %{ $ref } ) {
    $form->{$key} = $ref->{$key};
  }
  
  $sth->finish;
  
  # part or service item
  $form->{item} = ($form->{inventory_accno}) ? 'part' : 'service';
  if ($form->{assembly}) {
    $form->{item} = 'assembly';

    # retrieve assembly items
    $query = qq|SELECT parts.id, parts.partnumber, parts.description,
                parts.sellprice, parts.weight, assembly.qty
                FROM parts, assembly
		WHERE assembly.parts_id = parts.id
		AND assembly.id = $form->{id}
		ORDER BY partnumber|;

    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);
    
    $form->{rowcount} = 0;
    while (my $ref = $sth->fetchrow_hashref) {
      $form->{rowcount}++;
      foreach my $key ( keys %{ $ref } ) {
	$form->{"${key}_$form->{rowcount}"} = $ref->{$key};
      }
    }
    $sth->finish;
  }

  # setup accno hash for <option checked> {amount} is used in create_links
  $form->{amount}{$ref->{inventory_accno}} = $form->{inventory_accno};
  $form->{amount}{$ref->{income_accno}} = $form->{income_accno};
  $form->{amount}{$ref->{expense_accno}} = $form->{expense_accno};
  

  if ($form->{item} eq 'part') {
    # get makes
    if ($form->{makemodel}) {
      $query = qq|SELECT name FROM makemodel
                  WHERE parts_id = $form->{id}|;

      $sth = $dbh->prepare($query);
      $sth->execute || $form->dberror($query);
      
      my $i = 1;
      while (($form->{"make_$i"}, $form->{"model_$i"}) = split(/:/, $sth->fetchrow_array)) {
	$i++;
	$form->{rowcount}++;
      }
      $sth->finish;
      $form->{rowcount} = $i - 1;

    }
  }

  # now get accno for taxes
  $query = qq|SELECT accno FROM partstax, chart
	      WHERE partstax.chart_id = chart.id
	      AND partstax.parts_id = $form->{id}|;
  
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (($key) = $sth->fetchrow_array) {
    $form->{amount}{$key} = $key;
  }

  $sth->finish;

  $dbh->disconnect;
  
}


sub save {
  my ($self, $myconfig, $form) = @_;

  ($form->{inventory_accno}) = split(/--/, $form->{IC});
  ($form->{expense_accno}) = split(/--/, $form->{IC_expense});
  ($form->{income_accno}) = split(/--/, $form->{IC_income});

  # connect to database, turn off AutoCommit
  my $dbh = $form->dbconnect_noauto($myconfig);

  # save the part
  # make up a unique handle and store in partnumber field
  # then retrieve the record based on the unique handle to get the id
  # replace the partnumber field with the actual variable
  # add records for makemodel

  # if there is a $form->{id} then replace the old entry
  # delete all makemodel entries and add the new ones

  # escape '
  foreach my $key (qw(partnumber description notes unit)) {
    $form->{$key} =~ s/'/\\'/g;
  }

  # if no data set values to 0
  foreach $key (qw(inventory_accno income_accno expense_accno)) {
    $form->{$key} *= 1;
  }

  # undo amount formatting
  foreach $key (qw(rop weight listprice sellprice)) {
    $form->{$key} = $form->parse_amount($myconfig, $form->{$key});
  }
  
  # set date to NULL if nothing entered
  $form->{priceupdate} = ($form->{priceupdate}) ? qq|'$form->{priceupdate}'| : "NULL";
  
  $form->{makemodel} = (($form->{make_1}) || ($form->{model_1})) ? TRUE : FALSE;

  $form->{alternate} = FALSE;
  $form->{assembly} = ($form->{item} eq 'assembly') ? TRUE : FALSE;
  
  
  my $query, $sth, $uid;
  
  if ($form->{id}) {
    
    if ($form->{item} eq 'part') {
      # delete makemodel records
      $query = qq|DELETE FROM makemodel
		  WHERE parts_id = $form->{id}|;
      $dbh->do($query) || $form->dberror($query);
    }

    if ($form->{item} eq 'assembly') {
      # delete assembly records
      $query = qq|DELETE FROM assembly
		  WHERE id = $form->{id}|;
      $dbh->do($query) || $form->dberror($query);
    }
    
    # delete tax records
    $query = qq|DELETE FROM partstax
		WHERE parts_id = $form->{id}|;
    $dbh->do($query) || $form->dberror($query);

  } else {
    
    $uid = time;
    $uid .= $form->{login};

    $query = qq|INSERT INTO parts (partnumber)
                VALUES ('$uid')|;
    $dbh->do($query) || $form->dberror($query);

    $query = qq|SELECT id
                FROM parts
		WHERE partnumber = '$uid'|;
    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);

    ($form->{id}) = $sth->fetchrow_array;
    $sth->finish;
  }
    
  $query = qq|UPDATE parts SET 
	      partnumber = '$form->{partnumber}',
	      description = '$form->{description}',
	      makemodel = $form->{makemodel},
	      alternate = $form->{alternate},
	      assembly = $form->{assembly},
	      listprice = $form->{listprice},
	      sellprice = $form->{sellprice},
	      weight = $form->{weight},
	      priceupdate = $form->{priceupdate},
	      unit = '$form->{unit}',
	      notes = '$form->{notes}',
	      rop = $form->{rop},
	      inventory_accno_id = (SELECT id FROM chart
				    WHERE accno = $form->{inventory_accno}),
	      income_accno_id = (SELECT id FROM chart
				 WHERE accno = $form->{income_accno}),
	      expense_accno_id = (SELECT id FROM chart
				  WHERE accno = $form->{expense_accno})
	      WHERE id = $form->{id}|;
  $dbh->do($query) || $form->dberror($query);

 
  # insert makemodel records
  if ($form->{item} eq 'part') {
    for my $i (1 .. $form->{rowcount}) {
      # put make and model together
      if (($form->{"make_$i"}) || ($form->{"model_$i"})) {
	$form->{"make_$i"} =~ s/'/\\'/g;
	$form->{"model_$i"} =~ s/'/\\'/g;
	
	$query = qq|INSERT INTO makemodel (parts_id, name)
		    VALUES ($form->{id},
		    '$form->{"make_$i"}:$form->{"model_$i"}')|;
	$dbh->do($query) || $form->dberror($query);
      }
    }
  }


  # insert taxes
  foreach $item (split / /, $form->{taxaccounts}) {
    if ($form->{"IC_tax_$item"}) {
      $query = qq|INSERT INTO partstax (parts_id, chart_id)
                  VALUES ($form->{id}, 
		          (SELECT id
			   FROM chart
			   WHERE accno = $item))|;
      $dbh->do($query) || $form->dberror($query);
    }
  }

  # add assembly records
  if ($form->{item} eq 'assembly') {
    for my $i (1 .. $form->{rowcount}) {
      $form->{"qty_$i"} = $form->parse_amount($myconfig, $form->{"qty_$i"});
      
      if ($form->{"qty_$i"} != 0) {
	$query = qq|INSERT INTO assembly (id, parts_id, qty)
		    VALUES ($form->{id}, $form->{"id_$i"}, $form->{"qty_$i"})|;
	$dbh->do($query) || $form->dberror($query);
      }
    }
  }
      
  # commit
  my $rc = $dbh->commit;
  $dbh->disconnect;

  $rc;
  
}



sub delete {
  my ($self, $myconfig, $form) = @_;

  # connect to database, turn off AutoCommit
  my $dbh = $form->dbconnect_noauto($myconfig);

  # check if the item is in the invoice table
  my $query = qq|SELECT id
	      FROM invoice
	      WHERE parts_id = $form->{id}|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);
  my ($id) = $sth->fetchrow_array;
  $sth->finish;

  if ($id) {
    $dbh->disconnect;
    return 2;
  }
  
  # check if part belongs to an assembly
  $query = qq|SELECT id
              FROM assembly
	      WHERE parts_id = $form->{id}|;
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);
  ($id) = $sth->fetchrow_array;
  $sth->finish;

  if ($id) {
    $dbh->disconnect;
    return 3;
  }
  
  $query = qq|DELETE FROM parts
	      WHERE id = $form->{id}|;
  $dbh->do($query) || $form->dberror($query);

  $query = qq|DELETE FROM partstax
	      WHERE parts_id = $form->{id}|;
  $dbh->do($query) || $form->dberror($query);

  # check if it is a part, assembly or service
  if ($form->{item} eq 'part') {
    $query = qq|DELETE FROM makemodel
		WHERE parts_id = $form->{id}|;
    $dbh->do($query) || $form->dberror($query);
  }

  if ($form->{item} eq 'assembly') {
    $query = qq|DELETE FROM assembly
		WHERE id = $form->{id}|;
    $dbh->do($query) || $form->dberror($query);
  }
  
  if ($form->{item} eq 'alternate') {
    $query = qq|DELETE FROM alternate
		WHERE id = $form->{id}|;
    $dbh->do($query) || $form->dberror($query);
  }

  # commit
  my $rc = $dbh->commit;
  $dbh->disconnect;

  $rc;
  
}


sub assembly_item {
  my ($self, $myconfig, $form) = @_;

  my $i = $form->{rowcount};
  my $partnumber = lc $form->{"partnumber_$i"};
  my $description = lc $form->{"description_$i"};
  my $where = "true";

  if ($partnumber) {
    $where .= " AND lower(partnumber) LIKE '%$partnumber%'";
  }
  if ($description) {
    $where .= " AND lower(description) LIKE '%$description%'";
  }

  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $query = qq|SELECT id, partnumber, description, sellprice
		   FROM parts
		   WHERE $where
		   ORDER BY partnumber|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my $ref = $sth->fetchrow_hashref) {
    push @{ $form->{item_list} }, $ref;
  }
  
  $sth->finish;
  $dbh->disconnect;
  
}


sub all_parts {
  my ($self, $myconfig, $form) = @_;

  my $where = 'true';
  if ($form->{partnumber}) {
    my $partnumber = lc $form->{partnumber};
    $where .= " AND lower(partnumber) LIKE '%$partnumber%'";
  }
  if ($form->{description}) {
    my $description = lc $form->{description};
    $where .= " AND lower(description) LIKE '%$description%'";
  }
 
  if ($form->{searchitems} eq 'parts') {
    $where .= " AND inventory_accno_id > 0";
  }
  if ($form->{searchitems} eq 'assemblies') {
    $where .= " AND assembly";
  }
  if ($form->{searchitems} eq 'services') {
    $where .= " AND inventory_accno_id = NULL AND NOT assembly";
  }
  if ($form->{searchitems} eq 'onhand') {
    $where .= " AND onhand != 0";
  }

  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $query = qq|SELECT id, partnumber, description, onhand, unit, sellprice
                 FROM parts
  	         WHERE $where
	         ORDER BY $form->{sort}|;

  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my $ref = $sth->fetchrow_hashref) {
    push @{ $form->{parts} }, $ref;
  }

  $sth->finish;
  $dbh->disconnect;

}


sub create_links {
  my ($self, $module, $myconfig, $form) = @_;

  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $query = qq|SELECT accno, description, link
                 FROM chart
		 WHERE link LIKE '%$module%'
		 ORDER BY accno|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my $ref = $sth->fetchrow_hashref) {
    foreach my $key (split(/:/, $ref->{link})) {
      if ($key =~ /$module/) {
	push @{ $form->{"${module}_links"}{$key} }, { accno => $ref->{accno},
				      description => $ref->{description} };
      }
    }
  }

  $sth->finish;

  if ($form->{id}) {
    $query = qq|SELECT nativecurr, weightunit
                FROM defaults|;
    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);

    ($form->{nativecurr}, $form->{weightunit}) = $sth->fetchrow_array;
    $sth->finish;

  } else {
    $query = qq|SELECT nativecurr, weightunit, current_date
                FROM defaults|;
    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);

    ($form->{nativecurr}, $form->{weightunit}, $form->{priceupdate}) = $sth->fetchrow_array;
    $sth->finish;
  }
  
  $dbh->disconnect;

}


1;

