#=====================================================================
# SQL-Ledger Accounting
# Copyright (C) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
#
# Inventory received module
#
# CHANGE LOG
#   DS. 2000-07-04  Created
#   DS. 2001-06-13  Changed accno links to chart_id
#
#======================================================================

package IR;


sub post_invoice {
  my ($self, $myconfig, $form) = @_;
  
  # connect to database, turn off autocommit
  my $dbh = $form->dbconnect_noauto($myconfig);

  my $query, $uid, $sth;

  if ($form->{id}) {

    &reverse_invoice($dbh, $form);

  } else {
    # create AP entry
    $uid = time;
    $uid .= $form->{login};

    $query = qq|INSERT INTO ap (invnumber) values ('$uid')|;
    $dbh->do($query) || $form->dberror($query);

    # retrieve id
    $query = qq|SELECT id FROM ap WHERE invnumber = '$uid'|;
    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);

    ($form->{id}) = $sth->fetchrow_array;
    $sth->finish;
  }

  map { $form->{$_} =~ s/'/\\'/g } qw(invnumber ordnumber);
  
  my $netamount = 0;
  my $tax = 0;
  my $linetotal;
  my $taxrate;
  my $taxamount;

  for my $i (1 .. $form->{rowcount}) {
    
    $form->{"qty_$i"} = $form->parse_amount($myconfig, $form->{"qty_$i"});
    
    if ($form->{"qty_$i"} != 0) {
      
      map { $form->{"${_}_$i"} =~ s/'/\\'/g } qw(partnumber description unit);
      
      my $allocated = 0;
      
      # set values to 0 if nothing entered
      $form->{"sellprice_$i"} = $form->parse_amount($myconfig, $form->{"sellprice_$i"});
      $form->{"inventory_accno_$i"} *= 1;
      
      # add the part if it is a new item
      unless ($form->{"id_$i"}) {
	$uid = time;
	$uid .= $form->{login};

	$query = qq|INSERT INTO parts (partnumber) values ('$uid')|;
	$dbh->do($query) || $form->dberror($query);

	# retrieve id
	$query = qq|SELECT id FROM parts WHERE partnumber = '$uid'|;
	$sth = $dbh->prepare($query);
	$sth->execute || $form->dberror($query);

	($form->{"id_$i"}) = $sth->fetchrow_array;
	$sth->finish;
	
	$query = qq|UPDATE parts SET
		    partnumber = '$form->{"partnumber_$i"}',
		    description = '$form->{"description_$i"}',
		    unit = '$form->{"unit_$i"}',
	            inventory_accno_id = (SELECT id FROM chart
	                          WHERE accno = $form->{"inventory_accno_$i"}),
		    income_accno_id = (SELECT id FROM chart
		                  WHERE accno = $form->{"income_accno_$i"}),
		    expense_accno_id = (SELECT id FROM chart
		                  WHERE accno = $form->{"expense_accno_$i"})
	            WHERE id = $form->{"id_$i"}|;
	
	$dbh->do($query) || $form->dberror($query);

	# add taxes for the part
	foreach my $item (split / /, $form->{"taxaccounts_$i"}) {
          $query = qq|INSERT INTO partstax
	              (parts_id, chart_id)
		      VALUES ($form->{"id_$i"},
		              (SELECT id FROM chart
			       WHERE accno = '$item'))|;
          $dbh->do($query) || $form->dberror($query);
	}
      }


      if ($form->{"inventory_accno_$i"}) {
	
	# update onhand and lastcost
	$query = qq|SELECT onhand
		    FROM parts WHERE id = $form->{"id_$i"}|;
	$sth = $dbh->prepare($query);
	$sth->execute || $form->dberror($query);

	my ($onhand) = $sth->fetchrow_array;
	$sth->finish;

        $onhand += $form->{"qty_$i"};
	
	# update parts table
	$query = qq|UPDATE parts SET
		    lastcost = $form->{"sellprice_$i"},
		    onhand = $onhand
	            WHERE id = $form->{"id_$i"}|;
	$dbh->do($query) || $form->dberror($query);

	$linetotal = sprintf("%.2f", $form->{"qty_$i"} * $form->{"sellprice_$i"});
        
	$taxrate = 0;
	map { $taxrate += $form->{"${_}_rate"} } split / /, $form->{"taxaccounts_$i"};
	
	if ($form->{taxincluded}) {
	  $form->{"sellprice_$i"} = $form->{"sellprice_$i"} * (1 / (1 + $taxrate));
	  $taxamount = sprintf("%.2f", $linetotal - ($form->{"sellprice_$i"} * $form->{"qty_$i"}));
	} else {
	  $taxamount = sprintf("%.2f", $linetotal * $taxrate);
	}
	
	$tax += $taxamount;

	if ($taxamount != 0) {
	  map { $amount{$form->{id}}{$_} -= sprintf("%.2f", $taxamount * $form->{"${_}_rate"} / $taxrate) } split / /, $form->{"taxaccounts_$i"};
	}

	$linetotal = sprintf("%.2f", $form->{"sellprice_$i"} * $form->{"qty_$i"});
	# add purchase to inventory
	$amount{$form->{id}}{$form->{"inventory_accno_$i"}} -= $linetotal;
	
        $netamount += $linetotal;

        # check if we sold the item already and
        # make an entry for the expense and inventory
	$query = qq|SELECT i.id, i.qty, i.allocated, i.trans_id,
		    i.inventory_accno_id, i.expense_accno_id, a.transdate
		    FROM invoice i, ar a
	            WHERE i.parts_id = $form->{"id_$i"}
		    AND i.qty > 0
		    AND (i.qty + i.allocated) > 0
		    AND i.trans_id = a.id
		    ORDER BY id DESC|;
	$sth = $dbh->prepare($query);
	$sth->execute || $form->dberror($query);


        my $totalqty = $form->{"qty_$i"};
	
	while (my $ref = $sth->fetchrow_hashref) {
	  
	  my $qty = $ref->{qty} + $ref->{allocated};
	  
	  if (($qty - $totalqty) > 0) {
	    $qty = $totalqty;
	  }


          $linetotal = sprintf("%.2f", $form->{"sellprice_$i"} * $qty);
	  
	  if ($ref->{allocated} < 0) {
	    # we have an entry for it already, adjust amount
	    $form->update_balance($dbh,
				  "acc_trans",
				  "amount",
				  qq|trans_id = $ref->{trans_id} AND chart_id = $ref->{inventory_accno_id} AND transdate = '$ref->{transdate}'|,
				  $linetotal);

	    $form->update_balance($dbh,
				  "acc_trans",
				  "amount",
				  qq|trans_id = $ref->{trans_id} AND chart_id = $ref->{expense_accno_id} AND transdate = '$ref->{transdate}'|,
				  -($linetotal));

	  } else {
	    # add entry for inventory, this one is for the sold item
	    $query = qq|INSERT INTO acc_trans (trans_id, chart_id, amount, 
			transdate)
			VALUES ($ref->{trans_id}, $ref->{inventory_accno_id},
			$linetotal, '$ref->{transdate}')|;
	    $dbh->do($query) || $form->dberror($query);

	    # add expense
	    $query = qq|INSERT INTO acc_trans (trans_id, chart_id, amount, 
			transdate)
			VALUES ($ref->{trans_id}, $ref->{expense_accno_id},
			|. -($linetotal) .qq|, '$ref->{transdate}')|;
	    $dbh->do($query) || $form->dberror($query);
	  
	  }
	
	  # update allocated for sold item
	  $form->update_balance($dbh,
				"invoice",
				"allocated",
				qq|id = $ref->{id}|,
				-($qty));
	
	  $allocated += $qty;

	  last if (($totalqty -= $qty) <= 0);
	}

	$sth->finish;
	
      } else {
	
	# update lastcost
	$query = qq|UPDATE parts SET
		    lastcost = $form->{"sellprice_$i"}
	            WHERE id = $form->{"id_$i"}|;
	$dbh->do($query) || $form->dberror($query);


	$linetotal = sprintf("%.2f", $form->{"sellprice_$i"} * $form->{"qty_$i"});

	$taxrate = 0;
	map { $taxrate += $form->{"${_}_rate"} } split / /, $form->{"taxaccounts_$i"};

        if ($form->{taxincluded}) {
	  $form->{"sellprice_$i"} = $form->{"sellprice_$i"} * (1 / (1 + $taxrate));
	  $taxamount = sprintf("%.2f", $linetotal - ($form->{"sellprice_$i"} * $form->{"qty_$i"}));
	} else {
	  $taxamount = sprintf("%.2f", $linetotal * $taxrate);
	}

        if ($taxamount != 0) {
	  map { $amount{$form->{id}}{$_} -= sprintf("%.2f", $taxamount * $form->{"${_}_rate"} / $taxrate) } split / /, $form->{"taxaccounts_$i"};
	}

	$tax += $taxamount;

	$linetotal = sprintf("%.2f", $form->{"sellprice_$i"} * $form->{"qty_$i"});
        $netamount += $linetotal;

	# total expenses
	$amount{$form->{id}}{$form->{"expense_accno_$i"}} -= $linetotal;
      }

      
      # save detail record in invoice table
      $query = qq|INSERT INTO invoice
		 (trans_id, inventory_accno_id, income_accno_id,
		  expense_accno_id,
		  parts_id, partnumber, description, unit, qty, sellprice,
		  allocated)
		  VALUES ($form->{id},
		         (SELECT id FROM chart
		         WHERE accno = $form->{"inventory_accno_$i"}),
		         (SELECT id FROM chart
			 WHERE accno = $form->{"income_accno_$i"}),
			 (SELECT id FROM chart
			 WHERE accno = $form->{"expense_accno_$i"}),
		  $form->{"id_$i"}, '$form->{"partnumber_$i"}',
		  '$form->{"description_$i"}', '$form->{"unit_$i"}',
		  | . -($form->{"qty_$i"}) . qq|, $form->{"sellprice_$i"},
		  $allocated)|;
      $dbh->do($query) || $form->dberror($query);

    }
  }


  $form->{paid} = $form->parse_amount($myconfig, $form->{paid});
  $amount{$form->{id}}{$form->{AP}} = sprintf("%.2f", $netamount + $tax - $form->{paid});
  $amount{$form->{id}}{$form->{AP_paid}} = $form->{paid};
  
  
  # record acc_trans transactions and update chart
  foreach my $trans_id (keys %amount) {
    foreach my $accno (keys %{ $amount{$trans_id} }) {
      if (($amount{$trans_id}{$accno} = sprintf("%.2f", $amount{$trans_id}{$accno})) != 0) {
	$query = qq|INSERT INTO acc_trans (trans_id, chart_id, amount, 
		    transdate)
		    VALUES ($form->{id}, (SELECT id FROM chart
		                         WHERE accno = $accno),
		    $amount{$trans_id}{$accno}, '$form->{invdate}')|;
	$dbh->do($query) || $form->dberror($query);
      }
    }
  }

  # set values which could be empty
  $form->{taxincluded} = ($form->{taxincluded}) ? TRUE : FALSE;
  my $datepaid = ($form->{paid}) ? qq|'$form->{invdate}'| : "NULL";
  my $duedate = ($form->{duedate}) ? qq|'$form->{duedate}'| : "NULL";
  
  $netamount = sprintf("%.2f", $netamount);
  $amount = sprintf("%.2f", $netamount + $tax);

  # save AP record
  $query = qq|UPDATE ap set
              invnumber = '$form->{invnumber}',
	      ordnumber = '$form->{ordnumber}',
              transdate = '$form->{invdate}',
              vendor = $form->{vendor_id},
              amount = $amount,
              netamount = $netamount,
              paid = $form->{paid},
	      datepaid = $datepaid,
	      duedate = $duedate,
	      invoice = TRUE,
	      taxincluded = $form->{taxincluded}
              WHERE id = $form->{id}|;
  $dbh->do($query) || $form->dberror($query);

  my $rc = $dbh->commit;
  $dbh->disconnect;

  $rc;
  
}



sub reverse_invoice {
  my ($dbh, $form) = @_;
  
  # reverse inventory items
  my $query = qq|SELECT * FROM invoice
                 WHERE trans_id = $form->{id}|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  my $netamount = 0;
  
  while (my $ref = $sth->fetchrow_hashref) {
    $netamount += sprintf("%.2f", -($ref->{sellprice} * $ref->{qty}));

    if ($ref->{inventory_accno_id}) {
      # adjust onhand in parts table, check if part still exists
      $query = qq|SELECT id
                  FROM parts
		  WHERE id = $ref->{parts_id}|;
      my $pth = $dbh->prepare($query);
      $pth->execute || $form->dberror($query);

      if ($pth->fetchrow_array) {
	$form->update_balance($dbh,
			      "parts",
			      "onhand",
			      qq|id = $ref->{parts_id}|,
			      $ref->{qty});
      } else {
	
	map { $ref->{$_} =~ s/'/\\'/g } qw(partnumber description unit);

	$query = qq|INSERT INTO parts (id, partnumber, description, unit,
		    inventory_accno_id, income_accno_id, expense_accno_id,
		    onhand, listprice, sellprice, lastcost)
		    VALUES ($ref->{parts_id}, '$ref->{partnumber}',
		    '$ref->{description}', '$ref->{unit}',
		    $ref->{inventory_accno_id}, $ref->{income_accno_id},
		    $ref->{expense_accno_id}, $ref->{qty},
		    $ref->{sellprice}, $ref->{sellprice}, $ref->{sellprice})|;
	$dbh->do($query) || $form->dberror($query);
	
	# insert taxes
	my $taxrate = 0;
	foreach $item (split / /, $form->{taxaccounts}) {
	  $taxrate += $form->{"${item}_rate"};
	  $query = qq|INSERT INTO partstax (parts_id, chart_id)
	              VALUES ($ref->{parts_id}, 
		              (SELECT id FROM chart
			       WHERE accno = $item))|;
	  $dbh->do($query) || $form->dberror($query);
	}
      }
      
      $pth->finish;

      # if $ref->{allocated} > 0 than we sold that many items
      if ($ref->{allocated} > 0) {
	# get references for sold items
	$query = qq|SELECT i.id, i.trans_id, i.qty,
	            a.transdate
	            FROM invoice i, ar a
		    WHERE i.parts_id = $ref->{parts_id}
		    AND i.qty > 0
		    AND i.allocated < 0
		    AND i.trans_id = a.id
		    ORDER BY trans_id DESC|;
	$pth = $dbh->prepare($query);
	$pth->execute || $form->dberror($query);

	while ($pthref = $pth->fetchrow_hashref) {
	  my $qty = $ref->{allocated};
	  if (($qty - $pthref->{qty}) > 0) {
	    $qty = $pthref->{qty};
	  }

	  my $amount = sprintf("%.2f", $ref->{sellprice} * $qty);
	  
	  #adjust allocated
	  $form->update_balance($dbh,
				"invoice",
				"allocated",
				qq|id = $pthref->{id}|,
				$qty);
	  
	  $form->update_balance($dbh,
				"acc_trans",
				"amount",
				qq|trans_id = $pthref->{trans_id} AND chart_id = $ref->{expense_accno_id} AND transdate = '$pthref->{transdate}'|,
				$amount);
		      
	  $form->update_balance($dbh,
				"acc_trans",
				"amount",
				qq|trans_id = $pthref->{trans_id} AND chart_id = $ref->{inventory_accno_id} AND transdate = '$pthref->{transdate}'|,
				-($amount));

	  last if (($ref->{allocated} -= $qty) <= 0);
	}
	$pth->finish;
      }
    }
  }
  $sth->finish;


  # delete acc_trans
  $query = qq|DELETE FROM acc_trans
              WHERE trans_id = $form->{id}|;
  $dbh->do($query) || $form->dberror($query);

  # delete zero acc_trans transactions
  $query = qq|DELETE FROM acc_trans
              WHERE amount = 0|;
  $dbh->do($query) || $form->dberror($query);

  # delete invoice entries
  $query = qq|DELETE FROM invoice
              WHERE trans_id = $form->{id}|;
  $dbh->do($query) || $form->dberror($query);

} 



sub delete_invoice {
  my ($self, $myconfig, $form) = @_;

  # connect to database
  my $dbh = $form->dbconnect_noauto($myconfig);

  &reverse_invoice($dbh, $form);
  
  # delete AP record
  my $query = qq|DELETE FROM ap
                 WHERE id = $form->{id}|;
  $dbh->do($query) || $form->dberror($query);

  my $rc = $dbh->commit;
  $dbh->disconnect;

  $rc;
  
}


sub retrieve_invoice {
  my ($self, $myconfig, $form) = @_;
  
  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $query;

  if ($form->{id}) {
    # get default accounts and last invoice number
    $query = qq|SELECT (SELECT accno FROM chart
                       WHERE inventory_accno_id = chart.id)
                       AS inventory_accno,
		       (SELECT accno FROM chart
		       WHERE income_accno_id = chart.id)
		       AS income_accno,
		       (SELECT accno FROM chart
		       WHERE expense_accno_id = chart.id)
		       AS expense_accno,
                nativecurr
	 	FROM defaults|;
  } else {
    $query = qq|SELECT (SELECT accno FROM chart
                       WHERE inventory_accno_id = chart.id)
                       AS inventory_accno,
		       (SELECT accno FROM chart
		       WHERE income_accno_id = chart.id)
		       AS income_accno,
		       (SELECT accno FROM chart
		       WHERE expense_accno_id = chart.id)
		       AS expense_accno,
                ponumber AS invnumber, nativecurr, current_date AS invdate
                FROM defaults|;
  }
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  my $ref = $sth->fetchrow_hashref;
  map { $form->{$_} = $ref->{$_} } keys %$ref;
  $sth->finish;

  unless ($form->{id}) {

    # up invoice number by 1
    $form->{invnumber}++;

    # save the new number
    $query = qq|UPDATE defaults
                SET ponumber = '$form->{invnumber}'|;
    $dbh->do($query) || $form->dberror($query);
  }
  

  if ($form->{id}) {

    &get_vendor("", $myconfig, $form);
    
    # retrieve invoice
    $query = qq|SELECT invnumber, transdate as invdate, duedate,
                ordnumber, paid, taxincluded
		FROM ap WHERE id = $form->{id}|;
    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);

    map { $form->{$_} = $ref->{$_} } keys %{ $ref = $sth->fetchrow_hashref };
    $sth->finish;
    

    # retrieve individual items
    $query = qq|SELECT (SELECT accno FROM chart
                       WHERE inventory_accno_id = chart.id)
                       AS inventory_accno,
		       (SELECT accno FROM chart
		       WHERE income_accno_id = chart.id)
		       AS income_accno,
		       (SELECT accno FROM chart
		       WHERE expense_accno_id = chart.id)
		       AS expense_accno,
                partnumber, description, qty, sellprice, parts_id AS id, unit
		FROM invoice
		WHERE trans_id = $form->{id}
		ORDER BY id|;
    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);

    while ($ref = $sth->fetchrow_hashref) {

      # get tax rates for part
      $query = qq|SELECT accno
                  FROM chart
                  WHERE tax.chart_id = partstax.chart_id
	          AND partstax.chart_id = id
	          AND partstax.parts_id = $ref->{id}|;
      my $pth = $dbh->prepare($query);
      $pth->execute || $form->dberror($query);

      $ref->{taxaccounts} = "";
      my $taxrate = 0;
      
      while (my $ptref = $pth->fetchrow_hashref) {
        $ref->{taxaccounts} .= "$ptref->{accno} ";
        $taxrate += $form->{"$ptref->{accno}_rate"};
      }


      # if tax is included, add it
      if ($form->{taxincluded}) {
	$ref->{sellprice} *= (1 + $taxrate);
      }
      
      # format sellprice
      $ref->{sellprice} = sprintf("%.2f", $ref->{sellprice});
      
      $pth->finish;
      chop $ref->{taxaccounts};

      $ref->{qty} *= -1;
      
      push @{ $form->{invoice_details} }, $ref;
      
    }
    $sth->finish;
  }

  $dbh->disconnect;
  
}


sub get_vendor {
  my ($self, $myconfig, $form) = @_;
  
  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $duedate = ($form->{invdate}) ? "date '$form->{invdate}'" : "current_date";

  # get vendor
  my $query = qq|SELECT taxincluded, terms, $duedate + terms AS duedate
                 FROM vendor
	         WHERE id = $form->{vendor_id}|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  map { $form->{$_} = $ref->{$_} } keys %{ $ref = $sth->fetchrow_hashref };
  $sth->finish;
  
  # get taxes for vendor
  $query = qq|SELECT accno
              FROM chart
	      WHERE vendortax.chart_id = id
	      AND vendortax.vendor_id = $form->{vendor_id}|;
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  my $vendortax = ();
  while ($ref = $sth->fetchrow_hashref) {
    $vendortax{$ref->{accno}} = 1;
  }
  $sth->finish;


  # get tax rates and description
  $query = qq|SELECT c.accno, c.description, c.link, t.rate
              FROM chart c, tax t
              WHERE c.id = t.chart_id
	      AND c.link LIKE '%IC_tax%'
	      ORDER BY accno|;
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  $form->{taxaccounts} = "";
  $form->{taxpart} = "";
  $form->{taxservice} = "";
  while ($ref = $sth->fetchrow_hashref) {
    if ($vendortax{$ref->{accno}}) {
      $form->{"$ref->{accno}_rate"} = $ref->{rate};
      $form->{"$ref->{accno}_description"} = $ref->{description};
      $form->{taxaccounts} .= "$ref->{accno} ";
    }
    
    foreach my $item (split /:/, $ref->{link}) {
      if ($item =~ /IC_taxpart/) {
	$form->{taxpart} .= "$ref->{accno} ";
      }
      
      if ($item =~ /IC_taxservice/) {
	$form->{taxservice} .= "$ref->{accno} ";
      }
    }
  }
  $sth->finish;
  chop $form->{taxaccounts};
  chop $form->{taxpart};
  chop $form->{taxservice};

  $dbh->disconnect;
  
}


sub retrieve_item {
  my ($self, $myconfig, $form) = @_;

  my $i = $form->{rowcount};
  my $partnumber = lc $form->{"partnumber_$i"};
  my $description = lc $form->{"description_$i"};
  
  my $where = "true";
  
  if ($partnumber) {
    $where .= " AND lower(partnumber) LIKE '%$partnumber%'";
  }
  
  if ($description) {
    $where .= " AND lower(description) LIKE '%$description%'";
  }

  # don't include assemblies
  $where .= " AND NOT assembly";
  
  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $query = qq|SELECT id, partnumber, description, lastcost AS sellprice,
                        (SELECT accno FROM chart
		        WHERE inventory_accno_id = chart.id)
		        AS inventory_accno,
			(SELECT accno FROM chart
			WHERE income_accno_id = chart.id)
			AS income_accno,
			(SELECT accno FROM chart
			WHERE expense_accno_id = chart.id)
			AS expense_accno, unit
                 FROM parts
	         WHERE $where
	         ORDER BY partnumber|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my $ref = $sth->fetchrow_hashref) {
    # get tax rates for part
    $query = qq|SELECT chart.accno
		WHERE tax.chart_id = partstax.chart_id
		AND partstax.chart_id = chart.id
		AND partstax.parts_id = $ref->{id}|;
    my $pth = $dbh->prepare($query);
    $pth->execute || $form->dberror($query);

    $ref->{taxaccounts} = "";
    while (my $ptref = $pth->fetchrow_hashref) {
      $ref->{taxaccounts} .= "$ptref->{accno} ";
    }
    $pth->finish;
    chop $ref->{taxaccounts};
    
    push @{ $form->{item_list} }, $ref;
  }
  
  $sth->finish;
  $dbh->disconnect;
  
}



sub link_invoice {
  my ($self, $myconfig, $form) = @_;

  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $query = qq|SELECT vendor
                 FROM ap
                 WHERE id = $form->{id}|;
  my $sth = $dbh->prepare($query);

  $sth->execute || $form->dberror($query);

  ($form->{link}{vendor_id}) = $sth->fetchrow_array;

  $sth->finish;


  $query = qq|SELECT chart.accno, amount
              FROM acc_trans
	      WHERE acc_trans.chart_id = chart.id
              AND trans_id = $form->{id}|;
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my $ref = $sth->fetchrow_hashref) {
    $form->{amount}{$ref->{accno}} = $ref->{amount};
  }

  $sth->finish;
  $dbh->disconnect;

}


1;

