#=====================================================================
# SQL-Ledger Accounting
# Copyright (C) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
#
# Inventory invoicing module
#
# CHANGE LOG:
#   DS. 2000-07-04  Created
#   DS. 2001-06-13  Changed account links from accno to id
#
#======================================================================

package IS;


sub invoice_details {
  my ($self, $myconfig, $form) = @_;

  $form->{duedate} = $form->{invdate} unless ($form->{duedate});

  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $query = qq|SELECT (date '$form->{duedate}' - date '$form->{invdate}')|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  ($form->{terms}) = $sth->fetchrow_array;
  $sth->finish;
  $dbh->disconnect;
  
  my $tax = 0;
  
  foreach my $i (1 .. $form->{rowcount}) {
    $form->{"qty_$i"} = $form->parse_amount($myconfig, $form->{"qty_$i"});
    
    if ($form->{"qty_$i"} != 0) {

      # add number, description and qty to $form->{number}, ....
      push(@{ $form->{runningnumber} }, $i);
      push(@{ $form->{number} }, ($form->{"inventory_accno_$i"} || $form->{"assembly_$i"}) ? qq|$form->{"partnumber_$i"}| : " ");
      push(@{ $form->{description} }, qq|$form->{"description_$i"}|);
      push(@{ $form->{qty} }, $form->format_amount($myconfig, $form->{"qty_$i"}, "", "nat"));
      push(@{ $form->{unit} }, qq|$form->{"unit_$i"}|);

      
      push(@{ $form->{sellprice} }, $form->{"sellprice_$i"});
      
      # listprice
      push(@{ $form->{listprice} }, $form->{"listprice_$i"});

      my $discount = sprintf("%.2f", $form->parse_amount($myconfig, $form->{"sellprice_$i"}) * $form->parse_amount($myconfig, $form->{"discount_$i"}) / 100);
      
      # keep a netprice as well, (sellprice - discount)
      $form->{"netprice_$i"} = $form->parse_amount($myconfig, $form->{"sellprice_$i"}) - $discount;
      push(@{ $form->{netprice} }, ($form->{"netprice_$i"} != 0) ? $form->format_amount($myconfig, $form->{"netprice_$i"}) : " ");

      
      my $linetotal = sprintf("%.2f", $form->{"qty_$i"} * $form->{"netprice_$i"});

      $discount = ($discount > 0) ? $discount : " ";
      $linetotal = ($linetotal != 0) ? $linetotal : " ";
      
      push(@{ $form->{discount} }, $form->format_amount($myconfig, $discount));

      $form->{total} += $linetotal;

      push(@{ $form->{linetotal} }, $form->format_amount($myconfig, $linetotal));
      
      my $taxrate = 0;
      my $taxamount;
      my $taxbase;
      
      map { $taxrate += $form->{"${_}_rate"} } split / /, $form->{"taxaccounts_$i"};

      if ($form->{taxincluded}) {
	# calculate tax with unit price, rounding errors!
	my $sellprice = sprintf("%.2f", $form->parse_amount($myconfig, $form->{"sellprice_$i"}) * (1 / (1 + $taxrate)));
	$taxamount = sprintf("%.2f", $linetotal - $sellprice * $form->{"qty_$i"});
	$taxbase = $sellprice * $form->{"qty_$i"};
      } else {
        $taxamount = sprintf("%.2f", $linetotal * $taxrate);
	$taxbase = $linetotal;
	$tax += $taxamount;
      }
      
      if ($taxamount != 0) {
	foreach my $item (split / /, $form->{"taxaccounts_$i"}) {
	  $taxaccounts{$item} += sprintf("%.2f", $taxamount * $form->{"${item}_rate"} / $taxrate);
	  $taxbase{$item} += $taxbase;
	}
      }
    }
  }


  foreach my $item (sort keys %taxaccounts) {
    push(@{ $form->{taxbase} }, $form->format_amount($myconfig, $taxbase{$item}));
    push(@{ $form->{tax} }, $form->format_amount($myconfig, $taxaccounts{$item}));
    push(@{ $form->{taxdescription} }, $form->{"${item}_description"});
    push(@{ $form->{taxrate} }, $form->format_amount($myconfig, $form->{"${item}_rate"} * 100, "", "nat"));
    push(@{ $form->{taxnumber} }, $form->{"${item}_taxnumber"});
  }
    

  $form->{paid} = $form->parse_amount($myconfig, $form->{paid});

  $form->{subtotal} = $form->format_amount($myconfig, $form->{total}, 1);
  $form->{invtotal} = $form->format_amount($myconfig, $form->{total} + $tax, 1);
  $form->{total} = $form->format_amount($myconfig, $form->{total} + $tax - $form->{paid}, 1);
  $form->{paid} = $form->format_amount($myconfig, $form->{paid}, 1);


  # myconfig variables
  map { $form->{$_} = $myconfig->{$_} } (qw(company address tel fax signature businessnumber));
  $form->{username} = $myconfig->{name};

}


sub customer_details {
  my ($self, $myconfig, $form) = @_;
  
  # connect to database
  my $dbh = $form->dbconnect($myconfig);
  
  # get rest for the customer
  my $query = qq|SELECT name, addr1, addr2, addr3, addr4,
                 shiptoname, shiptoaddr1, shiptoaddr2, shiptoaddr3, shiptoaddr4,
	         phone as customerphone, fax as customerfax
	         FROM customer
	         WHERE id = $form->{customer_id}|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  map { $form->{$_} = $ref->{$_} } keys %{ $ref = $sth->fetchrow_hashref };

  $sth->finish;
  $dbh->disconnect;

}


sub post_invoice {
  my ($self, $myconfig, $form) = @_;
  
  # connect to database, turn off autocommit
  my $dbh = $form->dbconnect_noauto($myconfig);

  my $query, $sth, $uid;
  
  if ($form->{id}) {

    &reverse_invoice($dbh, $form);

  } else {
    # create AR entry
    $uid = time;
    $uid .= $form->{login};

    $query = qq|INSERT INTO ar (invnumber) values ('$uid')|;
    $dbh->do($query) || $form->dberror($query);

    # retrieve id
    $query = qq|SELECT id FROM ar WHERE invnumber = '$uid'|;
    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);

    ($form->{id}) = $sth->fetchrow_array;
    $sth->finish;
  }


  map { $form->{$_} =~ s/'/\\'/g } (qw(invnumber shippingpoint notes));

  my $netamount = 0;
  my $tax = 0;
  my $linetotal = 0;
  
  foreach my $i (1 .. $form->{rowcount}) {
    $form->{"qty_$i"} = $form->parse_amount($myconfig, $form->{"qty_$i"});
    
    if ($form->{"qty_$i"} != 0) {

      map { $form->{"${_}_$i"} =~ s/'/\\'/g } (qw(partnumber description unit));
      
      # calculate discount and set values to 0 if nothing entered
      $form->{"discount_$i"} = $form->parse_amount($myconfig, $form->{"discount_$i"}) / 100;
      if ($form->{"discount_$i"} > 1) {
	$form->{"discount_$i"} = 1;
      }

      $form->{"inventory_accno_$i"} *= 1;
      $form->{"expense_accno_$i"} *= 1;

      my $allocated = 0;
      my $taxrate = 0;
      my $taxamount;
      
      $form->{"sellprice_$i"} = sprintf("%.2f", $form->parse_amount($myconfig, $form->{"sellprice_$i"}) * (1 - $form->{"discount_$i"}));
      
      map { $taxrate += $form->{"${_}_rate"} } split / /, $form->{"taxaccounts_$i"};

      if ($form->{taxincluded}) {
	$form->{"sellprice_$i"} = $form->{"sellprice_$i"} * (1 / (1 + $taxrate));
      }
	
      $linetotal = sprintf("%.2f", $form->{"qty_$i"} * $form->{"sellprice_$i"});
      $taxamount = sprintf("%.2f", $linetotal * $taxrate);
      $tax += $taxamount;
      
      if ($taxamount != 0) {
	map { $form->{amount}{$form->{id}}{$_} += sprintf("%.2f", $taxamount * $form->{"${_}_rate"} / $taxrate) } split / /, $form->{"taxaccounts_$i"};
      }

      # add amount to income, $form->{amount}{trans_id}{accno}
      $form->{amount}{$form->{id}}{$form->{"income_accno_$i"}} += $linetotal;
      
      $netamount += $linetotal;


      if ($form->{"inventory_accno_$i"} || $form->{"assembly_$i"}) {
        # adjust parts onhand quantity
	$form->update_balance($dbh,
			      "parts",
			      "onhand",
			      qq|id = $form->{"id_$i"}|,
			      -($form->{"qty_$i"}));
     
        if ($form->{"assembly_$i"}) {
	  # record assembly item as allocated
	  &process_assembly($dbh, $form, $form->{"id_$i"}, $form->{"qty_$i"});
	} else {
	  $allocated = &allocate_expense($dbh, $form, $form->{"id_$i"}, $form->{"qty_$i"});
	}
      }

      # save detail record in invoice table
      $query = qq|INSERT INTO invoice
		 (trans_id, inventory_accno_id, income_accno_id,
		  expense_accno_id, parts_id, partnumber, description, unit,
		  qty, sellprice, discount, allocated, assemblyitem)
		  VALUES ($form->{id},
		      (SELECT id FROM chart
		       WHERE accno = $form->{"inventory_accno_$i"}),
		      (SELECT id FROM chart
		       WHERE accno = $form->{"income_accno_$i"}),
		      (SELECT id FROM chart
		       WHERE accno = $form->{"expense_accno_$i"}),
		  $form->{"id_$i"}, '$form->{"partnumber_$i"}',
		  '$form->{"description_$i"}', '$form->{"unit_$i"}',
		  $form->{"qty_$i"}, $form->{"sellprice_$i"},
		  $form->{"discount_$i"}, $allocated, FALSE)|;
      $dbh->do($query) || $form->dberror($query);
	
    }
  }


  $form->{paid} = $form->parse_amount($myconfig, $form->{paid});
  $form->{amount}{$form->{id}}{$form->{AR}} = -($netamount + $tax - $form->{paid});
  $form->{amount}{$form->{id}}{$form->{AR_paid}} = $form->{paid} * -1;
  

  foreach my $trans_id (keys %{$form->{amount}}) {
    foreach my $accno (keys %{ $form->{amount}{$trans_id} }) {
      if (($form->{amount}{$trans_id}{$accno} = sprintf("%.2f", $form->{amount}{$trans_id}{$accno})) != 0) {
	$query = qq|INSERT INTO acc_trans (trans_id,chart_id,amount,transdate)
		    VALUES ($trans_id, (SELECT id FROM chart
		                        WHERE accno = $accno),
		    $form->{amount}{$trans_id}{$accno}, '$form->{invdate}')|;
	$dbh->do($query) || $form->dberror($query);
      }
    }
  }


  $netamount = sprintf("%.2f", $netamount);
  $amount = $netamount + $tax;
  
  # set values which could be empty to 0
  $form->{terms} *= 1;
  $form->{taxincluded} = ($form->{taxincluded}) ? TRUE : FALSE;
  my $datepaid = ($form->{paid}) ? qq|'$form->{invdate}'| : "NULL";
  my $duedate = ($form->{duedate}) ? qq|'$form->{duedate}'| : "NULL";

  # save AR record
  my $amount = sprintf("%.2f", $netamount + $tax);
  
  $query = qq|UPDATE ar set
              invnumber = '$form->{invnumber}',
              transdate = '$form->{invdate}',
              customer = $form->{customer_id},
              amount = $amount,
              netamount = $netamount,
              paid = $form->{paid},
	      datepaid = $datepaid,
	      duedate = $duedate,
	      invoice = TRUE,
	      shippingpoint = '$form->{shippingpoint}',
	      terms = $form->{terms},
	      notes = '$form->{notes}',
	      taxincluded = $form->{taxincluded}
              WHERE id = $form->{id}
             |;
  $dbh->do($query) || $form->dberror($query);

  my $rc = $dbh->commit;
  $dbh->disconnect;

  $rc;
  
}


sub process_assembly {
  my ($dbh, $form, $id, $totalqty) = @_;

  my $query = qq|SELECT a.parts_id, a.qty, p.assembly,
                 p.partnumber, p.description, p.unit,
                 p.inventory_accno_id, p.income_accno_id,
		 p.expense_accno_id
                 FROM assembly a, parts p
		 WHERE a.parts_id = p.id
		 AND a.id = $id|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  
  while (my $ref = $sth->fetchrow_hashref) {

    $ref->{inventory_accno_id} *= 1;
    $ref->{expense_accno_id} *= 1;
    
    my $allocated = 0;
    
    map { $ref->{$_} =~ s/'/\\'/g } (qw(partnumber description unit));
    
    # multiply by number of assemblies
    $ref->{qty} *= $totalqty;
    
    if ($ref->{assembly}) {
      &process_assembly($dbh, $form, $ref->{parts_id}, $ref->{qty});
      next;
    } else {
      $allocated = &allocate_expense($dbh, $form, $ref->{parts_id}, $ref->{qty}) if ($ref->{inventory_accno_id});
    }

    # save detail record for individual assembly item in invoice table
    $query = qq|INSERT INTO invoice
	       (trans_id, partnumber, description, unit,
	        inventory_accno_id, income_accno_id,
		expense_accno_id, parts_id, qty, sellprice, allocated,
		assemblyitem)
		VALUES
		($form->{id}, '$ref->{partnumber}', '$ref->{description}',
		'$ref->{unit}',
		$ref->{inventory_accno_id}, $ref->{income_accno_id},
		$ref->{expense_accno_id},
		$ref->{parts_id}, $ref->{qty}, 0, $allocated, TRUE)|;
    $dbh->do($query) || $form->dberror($query);
	 
  }

  $sth->finish;

}


sub allocate_expense {
  my ($dbh, $form, $id, $totalqty) = @_;
    
  my $query = qq|SELECT *, (SELECT accno FROM chart
			 WHERE inventory_accno_id = chart.id)
			 AS inventory_accno,
			(SELECT accno FROM chart
			 WHERE income_accno_id = chart.id)
			 AS income_accno,
			(SELECT accno FROM chart
			 WHERE expense_accno_id = chart.id)
			 AS expense_accno
		  FROM invoice
		  WHERE parts_id = $id
		  AND qty < 0
		  AND (qty + allocated) < 0
		  ORDER BY id|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  my $qty;
  my $allocated = 0;

  while (my $ref = $sth->fetchrow_hashref) {
    if (($qty = (-($ref->{qty}) - $ref->{allocated})) > $totalqty) {
      $qty = $totalqty;
    }

    $form->update_balance($dbh,
			  "invoice",
			  "allocated",
			  qq|id = $ref->{id}|,
			  $qty);

    # total expenses and inventory
    # sellprice is the cost of the item
    $linetotal = sprintf("%.2f", $ref->{sellprice} * $qty);
    
    # add to expense
    $form->{amount}{$form->{id}}{$ref->{expense_accno}} += -$linetotal;

    # deduct inventory
    $form->{amount}{$form->{id}}{$ref->{inventory_accno}} -= -$linetotal;

    # add allocated
    $allocated += -$qty;
    
    last if (($totalqty -= $qty) <= 0);
  }

  $sth->finish;

  $allocated;
  
}



sub reverse_invoice {
  my ($dbh, $form) = @_;
  
  # reverse inventory items
  my $query = qq|SELECT *, parts.assembly
                 FROM invoice
                 WHERE invoice.parts_id = parts.id
		 AND trans_id = $form->{id}|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my $ref = $sth->fetchrow_hashref) {

    if ($ref->{inventory_accno_id} || $ref->{assembly}) {

      # if the invoice item belongs to don't adjust parts onhand
      unless ($ref->{assemblyitem}) {
	# adjust onhand in parts table
	$form->update_balance($dbh,
			      "parts",
			      "onhand",
			      qq|id = $ref->{parts_id}|,
			      $ref->{qty});
      }

      # if it is an assembly loop
      next if ($ref->{assembly});
      
      # adjust allocated
      $query = qq|SELECT * FROM invoice
		  WHERE parts_id = $ref->{parts_id}
		  AND qty < 0
		  AND allocated > 0
		  ORDER BY id DESC|;
      my $inh = $dbh->prepare($query);
      $inh->execute || $form->dberror($query);

      while (my $inhref = $inh->fetchrow_hashref) {
	if (($qty = $ref->{qty} - $inhref->{allocated}) > 0) {
	  $qty = $inhref->{allocated};
	} else {
	  $qty = $ref->{qty};
	}
	
	# update invoice
	$form->update_balance($dbh,
			      "invoice",
			      "allocated",
			      qq|id = $inhref->{id}|,
			      -$qty);

        last if (($ref->{qty} -= $qty) <= 0);
      }
      
      $inh->finish;
      
    }
  }
  
  $sth->finish;

  # delete acc_trans
  $query = qq|DELETE FROM acc_trans
              WHERE trans_id = $form->{id}|;
  $dbh->do($query) || $form->dberror($query);

  # delete invoice entries
  $query = qq|DELETE FROM invoice
              WHERE trans_id = $form->{id}|;
  $dbh->do($query) || $form->dberror($query);
  
}



sub delete_invoice {
  my ($self, $myconfig, $form) = @_;

  # connect to database
  my $dbh = $form->dbconnect_noauto($myconfig);

  &reverse_invoice($dbh, $form);
  
  # delete AR record
  my $query = qq|DELETE FROM ar
                 WHERE id = $form->{id}|;
  $dbh->do($query) || $form->dberror($query);

  my $rc = $dbh->commit;
  $dbh->disconnect;
  
  $rc;
  
}



sub retrieve_invoice {
  my ($self, $myconfig, $form) = @_;
  
  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $query;
  
  if ($form->{id}) {
    # get default accounts and last invoice number
    $query = qq|SELECT (SELECT accno FROM chart
                        WHERE inventory_accno_id = chart.id)
                        AS inventory_accno,
			(SELECT accno FROM chart
			WHERE income_accno_id = chart.id)
			AS income_accno,
			(SELECT accno FROM chart
			WHERE expense_accno_id = chart.id)
			AS expense_accno,
                nativecurr
		FROM defaults|;
  } else {
    $query = qq|SELECT (SELECT accno FROM chart
                        WHERE inventory_accno_id = chart.id)
                        AS inventory_accno,
			(SELECT accno FROM chart
			WHERE income_accno_id = chart.id)
			AS income_accno,
			(SELECT accno FROM chart
			WHERE expense_accno_id = chart.id)
			AS expense_accno,
                invnumber, nativecurr, current_date AS invdate
                FROM defaults|;
  }
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  my $ref = $sth->fetchrow_hashref;
  foreach my $key (keys %$ref) {
    $form->{$key} = $ref->{$key};
  }
  $sth->finish;

  unless ($form->{id}) {
    $form->{shippingpoint} = $myconfig->{shippingpoint};

    # up invoice number by 1
    $form->{invnumber}++;

    # save the new number
    $query = qq|UPDATE defaults
                SET invnumber = '$form->{invnumber}'|;
    $dbh->do($query) || $form->dberror($query);
  }
  
  
  
  if ($form->{id}) {
    
    &get_customer("", $myconfig, $form);
    
    # retrieve invoice
    $query = qq|SELECT invnumber, transdate as invdate, paid, shippingpoint,
		terms, notes, duedate, taxincluded
		FROM ar
		WHERE id = $form->{id}|;
    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);

    map { $form->{$_} = $ref->{$_} } keys %{ $ref = $sth->fetchrow_hashref };
    $sth->finish;
    

    # retrieve individual items
    $query = qq|SELECT (SELECT accno FROM chart
                       WHERE inventory_accno_id = chart.id)
                       AS inventory_accno,
		       (SELECT accno FROM chart
		       WHERE income_accno_id = chart.id)
		       AS income_accno,
		       (SELECT accno FROM chart
		       WHERE expense_accno_id = chart.id)
		       AS expense_accno,
                partnumber, description, qty, sellprice, discount,
		parts_id AS id, unit, parts.assembly
		FROM invoice
		WHERE trans_id = $form->{id}
		AND invoice.parts_id = parts.id
		AND NOT assemblyitem|;
    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);

    while ($ref = $sth->fetchrow_hashref) {
      # get taxes
      $query = qq|SELECT chart.accno
		  WHERE partstax.chart_id = chart.id
		  AND partstax.parts_id = $ref->{id}|;
      my $pth = $dbh->prepare($query);
      $pth->execute || $form->dberror($query);

      $ref->{taxaccounts} = "";
      my $taxrate = 0;
      
      while (my $ptref = $pth->fetchrow_hashref) {
	$ref->{taxaccounts} .= "$ptref->{accno} ";
	$taxrate += $form->{"$ptref->{accno}_rate"};
      }
      $pth->finish;
      chop $ref->{taxaccounts};

      # if tax is included, add it
      if ($form->{taxincluded}) {
	$ref->{sellprice} *= (1 + $taxrate);
      }

      # format sellprice
      # add discount
      $ref->{sellprice} = sprintf("%.2f", $ref->{sellprice} / (1 - $ref->{discount}));
      
      push @{ $form->{invoice_details} }, $ref;
    }
    $sth->finish;
  }

  $dbh->disconnect;
  
}


sub get_customer {
  my ($self, $myconfig, $form) = @_;
  
  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $duedate = ($form->{invdate}) ? "date '$form->{invdate}'" : "current_date";
  
  # get customer
  my $query = qq|SELECT discount, creditlimit, terms,
                 email, taxincluded,
	         $duedate + terms AS duedate
                 FROM customer
	         WHERE id = $form->{customer_id}|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  map { $form->{$_} = $ref->{$_} } keys %{ $ref = $sth->fetchrow_hashref };
  $sth->finish;

  # get taxes we charge for this customer
  $query = qq|SELECT accno
              FROM chart
	      WHERE customertax.chart_id = id
	      AND customertax.customer_id = $form->{customer_id}|;
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);
  
  my $customertax = ();
  while ($ref = $sth->fetchrow_hashref) {
    $customertax{$ref->{accno}} = 1;
  }
  $sth->finish;
    
  # get tax rates and description
  $query = qq|SELECT chart.accno, chart.description, tax.rate, tax.taxnumber
	      FROM chart, tax
	      WHERE chart.id = tax.chart_id
	      AND chart.link LIKE '%IC_tax%'
	      ORDER BY accno|;
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  $form->{taxaccounts} = "";
  while ($ref = $sth->fetchrow_hashref) {
    if ($customertax{$ref->{accno}}) {
      $form->{"$ref->{accno}_rate"} = $ref->{rate};
      $form->{"$ref->{accno}_description"} = $ref->{description};
      $form->{"$ref->{accno}_taxnumber"} = $ref->{taxnumber};
      $form->{taxaccounts} .= "$ref->{accno} ";
    }
  }
  $sth->finish;
  chop $form->{taxaccounts};

  $dbh->disconnect;
  
}



sub retrieve_item {
  my ($self, $myconfig, $form) = @_;

  my $i = $form->{rowcount};
  my $partnumber = lc $form->{"partnumber_$i"};
  my $description = lc $form->{"description_$i"};
  my $where = "true";

  if ($partnumber) {
    $where .= " AND lower(partnumber) LIKE '%$partnumber%'";
  }
  if ($description) {
    $where .= " AND lower(description) LIKE '%$description%'";
  }

  
  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $query = qq|SELECT
                 id, partnumber, description, sellprice, listprice,
                        (SELECT accno FROM chart
			WHERE inventory_accno_id = chart.id)
			AS inventory_accno,
			(SELECT accno FROM chart
			WHERE income_accno_id = chart.id)
			AS income_accno,
			(SELECT accno FROM chart
			WHERE expense_accno_id = chart.id)
			AS expense_accno,
		 unit, assembly
                 FROM parts
	         WHERE $where
	         ORDER BY partnumber|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my $ref = $sth->fetchrow_hashref) {

    # get taxes for part
    $query = qq|SELECT chart.accno
                FROM chart
		WHERE chart.id = partstax.chart_id
		AND partstax.parts_id = $ref->{id}|;
    my $pth = $dbh->prepare($query);
    $pth->execute || $form->dberror($query);

    $ref->{taxaccounts} = "";
    while (my $ptref = $pth->fetchrow_hashref) {
      $ref->{taxaccounts} .= "$ptref->{accno} ";
    }
    $pth->finish;
    chop $ref->{taxaccounts};

    push @{ $form->{item_list} }, $ref;

  }
  
  $sth->finish;
  $dbh->disconnect;
  
}


sub invoice_links {
  my ($self, $myconfig, $form) = @_;

  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $query = qq|SELECT customer FROM ar
                 WHERE id = $form->{id}|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  ($form->{customer_id}) = $sth->fetchrow_array;

  $sth->finish;


  $query = qq|SELECT chart.accno, acc_trans.amount
              WHERE acc_trans.chart_id = chart.id
              AND acc_trans.trans_id = $form->{id}|;
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my ($accno, $amount) = $sth->fetchrow_array) {
    $form->{amount}{$accno} = $amount;
  }
  $sth->finish;
  $dbh->disconnect;

}



sub retrieve_assemblies {
  my ($self, $myconfig, $form) = @_;
  
  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $where = 'true';
  my $partnumber = lc $form->{partnumber};
  my $description = lc $form->{description};
  
  if ($partnumber) {
    $where .= " AND lower(partnumber) LIKE '%$partnumber%'";
  }
  
  if ($description) {
    $where .= " AND lower(description) LIKE '%$description%'";
  }


  # retrieve assembly items
  my $query = qq|SELECT id, partnumber, description, onhand, rop
                 FROM parts
 		 WHERE $where
		 AND assembly|;

  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);
  
  while (my $ref = $sth->fetchrow_hashref) {
    push @{ $form->{assembly_items} }, $ref;
  }
  $sth->finish;

  $dbh->disconnect;
  
}


sub restock_assemblies {
  my ($self, $myconfig, $form) = @_;

  # connect to database
  my $dbh = $form->dbconnect_noauto($myconfig);
  
  for my $i (1 .. $form->{rowcount}) {

    $form->{"qty_$i"} *= 1;
    if ($form->{"qty_$i"} != 0) {
      &allocate_inventory($dbh, $form, $form->{"id_$i"}, $form->{"qty_$i"});
    }

  }
  
  my $rc = $dbh->commit;
  $dbh->disconnect;

  $rc;

}


sub allocate_inventory {
  my ($dbh, $form, $id, $qty) = @_;

  my $query = qq|SELECT id, inventory_accno_id, assembly, assembly.qty
		 FROM parts
		 WHERE assembly.parts_id = id
		 AND assembly.id = $id|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my $ref = $sth->fetchrow_hashref) {

    my $allocate = $qty * $ref->{qty};
    
    if ($ref->{assembly}) {
      &allocate_inventory($dbh, $form, $ref->{id}, $allocate);
    }

    
    # is it a service item, then loop
    $ref->{inventory_accno_id} *= 1;
    if ($ref->{inventory_accno_id} == 0) {
      next unless $ref->{assembly};
    }
    
    
    $allocate *= -1;
    
    # adjust parts onhand
    $form->update_balance($dbh,
			  "parts",
			  "onhand",
			  qq|id = $ref->{id}|,
			  $allocate);

  }

  $sth->finish;

  # update assembly
  $form->update_balance($dbh,
			"parts",
			"onhand",
			qq|id = $id|,
			$qty);

 
}


1;

