#=====================================================================
# SQL-Ledger, Accounting
# Copyright (c) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
#
# Inventory received module
#
# CHANGE LOG:
#   DS. 2000-07-04  Created
#   DS. 2001-07-25  added number formatting
#
#======================================================================


use DBI;
use SL::Form;
use SL::IR;

$form = new Form;


if ($form->{action}) {
  require("$userspath/$form->{login}.conf");
  $myconfig{dbpasswd} = unpack 'u', $myconfig{dbpasswd};
  map { $form->{$_} = $myconfig{$_} } qw(stylesheet charset);

  $locale = new Locale $myconfig{countrycode}, "ir";

  &{ $locale->findsub($form->{action}) };
  
} else {
  $form->error("action= not defined!");
}

# end of main



sub add {

  $form->{title} = $locale->text('Add Vendor Invoice');

  $form->{callback} = "$form->{script}?action=add&path=$form->{path}&login=$form->{login}";

  &create_links;
  
  &prepare_invoice;
  
}


sub continue {

  &{ $form->{nextsub} };

}


sub create_links {

  # create links
  $form->create_links("AP", \%myconfig, "vendor");

  # build the popup menus
  foreach $ref (@{ $form->{all_vendor} }) {
    if ($ref->{id} == $form->{vendor_id}) {
      $form->{vendor} = "$ref->{name}--$ref->{id}";
      IR->get_vendor(\%myconfig, \%$form);
    }
    $form->{selectvendor} .= "<option>$ref->{name}--$ref->{id}\n";
  }

  foreach $key (keys %{ $form->{AP_links} }) {
    foreach $ref (@{ $form->{AP_links}{$key} }) {
      if ($form->{amount}{$ref->{accno}}) {
	if ($key eq 'AP_paid') {
	  $form->{AP_paid} = $form->{amount}{$ref->{accno}};
	}
      }
      $form->{"select$key"} .= "<option>$ref->{accno}--$ref->{description}\n";
    }
  }

}



sub prepare_invoice {
  
  
  # get invoice
  IR->retrieve_invoice(\%myconfig, \%$form);


  if ($form->{id}) {
        
    map { $form->{$_} =~ s/"/&quot;/g } qw(invnumber ordnumber);

    foreach $ref (@{ $form->{invoice_details} }) {
      $i++;
      map { $form->{"${_}_$i"} = $ref->{$_} } keys %{ $ref };
      $form->{"sellprice_$i"} = $form->format_amount(\%myconfig, $form->{"sellprice_$i"});
      $form->{"qty_$i"} = $form->format_amount(\%myconfig, $form->{"qty_$i"});

      $form->{rowcount} = $i;
    }
  }

  $form->{new_invoice} = !$form->{id};
  
  &display_form;

}



sub form_header {

  $form->header;

  $form->{taxincluded} = ($form->{taxincluded}) ? "checked" : "";

  # set option selected
  map { $form->{$_} =~ s/ selected// } (qw(selectAP selectAP_paid selectvendor));
  map { $form->{"select${_}"} =~ s/option>$form->{$_}/option selected>$form->{$_}/ } (qw(AP AP_paid vendor));
    

  print qq|

<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<input type=hidden name=id value=$form->{id}>
<input type=hidden name=new_invoice value=$form->{new_invoice}>
<input type=hidden name=title value="$form->{title}">

<input type=hidden name=vendor_id value=$form->{vendor_id}>

<input type=hidden name=inventory_accno value=$form->{inventory_accno}>
<input type=hidden name=income_accno value=$form->{income_accno}>
<input type=hidden name=expense_accno value=$form->{expense_accno}>

<input type=hidden name=nativecurr value="$form->{nativecurr}">

<table border=1>
<tr valign=top>
<td>
<table callspacing=0 cellpadding=0>

<tr>
  <td colspan=5><b>|.$locale->text('Vendor').qq|</b> <select name=vendor>$form->{selectvendor}</select>
  </td>
</tr>

<input type=hidden name=selectvendor value="$form->{selectvendor}">

<tr>
  <td colspan=5><b>|.$locale->text('Record in').qq|</b>
  <select name=AP>$form->{selectAP}</select>
  <b>|.$locale->text('Tax Included').qq|</b>
  <input name=taxincluded class=checkbox type=checkbox value=1 $form->{taxincluded}>
  </td>
  <input type=hidden name=selectAP value="$form->{selectAP}">
</tr>

<tr>
  <td colspan=5 nowrap>
  <b>|.$locale->text('Invoice Number').qq|</b> 
  <font size=-1> <input name=invnumber size=10 value="$form->{invnumber}"></font>
  <b>|.$locale->text('Order Number').qq|</b>
  <font size=-1> <input name=ordnumber size=10 value="$form->{ordnumber}"></font>
  </td>
</tr>

<tr>
  <td colspan=5 nowrap>
  <b>|.$locale->text('Invoice Date').qq| ($myconfig{dateformat})</b> 
  <font size=-1> <input name=invdate size=10 value=$form->{invdate}></font>
  
  <b>|.$locale->text('Date Due').qq| ($myconfig{dateformat})</b> 
  <font size=-1> <input name=duedate size=10 value=$form->{duedate}></font>
  </td>
</tr>

<input type=hidden name=taxpart value="$form->{taxpart}">
<input type=hidden name=taxservice value="$form->{taxservice}">

<input type=hidden name=taxaccounts value="$form->{taxaccounts}">
|;

  foreach $item (split / /, $form->{taxaccounts}) {
    print qq|
<input type=hidden name="${item}_rate" value=$form->{"${item}_rate"}>
<input type=hidden name="${item}_description" value="$form->{"${item}_description"}">
|;
  }

}



sub form_footer {

  unless ($form->{taxincluded}) {

    $form->{invsubtotal} = $form->format_amount(\%myconfig, $form->{invsubtotal});
    
    $subtotal = qq|
      <tr>
        <th align=right>|.$locale->text('Subtotal').qq|:</th>
        <td align=right>$form->{invsubtotal}</td>
      </tr>
      |;

    foreach $item (split / /, $form->{taxaccounts}) {
      if ($form->{"${item}_total"}) {
	$form->{"${item}_total"} = $form->format_amount(\%myconfig, $form->{"${item}_total"});
	$tax .= qq|
	<tr>
	  <th align=right>$form->{"${item}_description"}:</th>
	  <td align=right>$form->{"${item}_total"}</td>
	  </tr>
	|;
      }
    }
  }


  $form->{invtotal} = $form->format_amount(\%myconfig, $form->{invtotal});

  print qq|

<input name=callback type=hidden value="$form->{callback}">

<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

</table>
</td>
</tr>

<tr>
  <td colspan=5>
    <table width=100%>
    <tr>
      <td>
      <table>
        $subtotal

	$tax
	
	<tr>
	  <th align=right>|.$locale->text('Total').qq|:</th>
	  <td>$form->{invtotal}</td>
        </tr>
      </table>
      </td>

      <th align=right>|.$locale->text('Paid').qq|</th>
      <td><font size=-1><input name=paid size=10 value=$form->{paid}></font></td>
      <td><select name=AP_paid>$form->{selectAP_paid}</select></td>
      <input name=selectAP_paid type=hidden value="$form->{selectAP_paid}">
    </tr>
    </table>
  </td>
</tr>

</table>

<input class=submit type=submit name=action value="|.$locale->text('Update Form').qq|">
<input class=submit type=submit name=action value="|.$locale->text('Post Invoice').qq|">
|;

  if ($form->{id}) {
    print qq|<input class=submit type=submit name=action value="|.$locale->text('Delete Invoice').qq|">|;
  }

print qq|
</form>
|;

}



sub display_form {

  &form_header;

  $form->{rowcount}++;
  
  # create rows
  map { &display_row($_) } (1 .. $form->{rowcount});
  
  &form_footer;
}



sub display_row {
  my $i = shift;

  if ($i == 1) {
    $form->{invsubtotal} = 0;
    $form->{invtotal} = 0;
  
    map { $form->{"${_}_total"} = 0 } split / /, $form->{taxaccounts};

    print qq|
<tr bgcolor=336666>
  <th><font color=ffffff>|.$locale->text('Number').qq|</th>
  <th><font color=ffffff>|.$locale->text('Description').qq|</th>
  <th><font color=ffffff>|.$locale->text('Qty').qq|</th>
  <th><font color=ffffff>|.$locale->text('Unit').qq|</th>
  <th><font color=ffffff>|.$locale->text('Price').qq|</th>
  <th><font color=ffffff>|.$locale->text('Amount').qq|</th>
</tr>
|;
  }

  $form->{"sellprice_$i"} = $form->parse_amount(\%myconfig, $form->{"sellprice_$i"});
  $form->{"qty_$i"} = $form->parse_amount(\%myconfig, $form->{"qty_$i"});
  
  $linetotal = $form->format_amount(\%myconfig, $form->{"sellprice_$i"} * $form->{"qty_$i"});

  print qq|
<tr>
  <td><font size=-1><input name="partnumber_$i" size=10 value="$form->{"partnumber_$i"}"></td>
  <td><font size=-1><input name="description_$i" size=30 value="$form->{"description_$i"}"></td>
  <td><font size=-1><input name="qty_$i" size=3 value=|.$form->format_amount(\%myconfig, $form->{"qty_$i"}, "", "nat").qq|></td>
  <td><font size=-1><input name="unit_$i" size=4 value="$form->{"unit_$i"}"></td>
  <td><font size=-1><input name="sellprice_$i" size=7 value=|.$form->format_amount(\%myconfig, $form->{"sellprice_$i"}).qq|></td>
  <td align=right><font size=-1>$linetotal</td>
</tr>

<input type=hidden name="id_$i" value=$form->{"id_$i"}>
<input type=hidden name="inventory_accno_$i" value=$form->{"inventory_accno_$i"}>
<input type=hidden name="income_accno_$i" value=$form->{"income_accno_$i"}>
<input type=hidden name="expense_accno_$i" value=$form->{"expense_accno_$i"}>

<input type=hidden name="taxaccounts_$i" value="$form->{"taxaccounts_$i"}">

<input type=hidden name=rowcount value=$i>

|;

  $linetotal = sprintf("%.2f", $form->{"sellprice_$i"} * $form->{"qty_$i"});
  my $taxtotal = 0;
  my $taxrate = 0;

  map { $taxrate += $form->{"${_}_rate"} } split / /, $form->{"taxaccounts_$i"};
  
  if ($form->{taxincluded}) {
    $taxtotal = sprintf("%.2f", $linetotal * (1 - 1 / (1 + $taxrate)));
  } else {
    $taxtotal = sprintf("%.2f", $linetotal * $taxrate);
  }

  if ($taxtotal != 0) {
    foreach my $item (split / /, $form->{"taxaccounts_$i"}) {
      $form->{"${item}_total"} += sprintf("%.2f", $taxtotal * $form->{"${item}_rate"}/$taxrate);
      $form->{"${item}_total"} = sprintf("%.2f", $form->{"${item}_total"});
    }
  }
  
  $form->{invsubtotal} += $linetotal;

  if ($form->{taxincluded}) {
    $form->{invtotal} = $form->{invsubtotal};
  } else {
    $form->{invtotal} += $linetotal + $taxtotal;
  }

}



sub update_form {

  # if the vendor changed get new values
  ($null, $vendor_id) = split /--/, $form->{vendor};
  if ($form->{vendor_id} ne $vendor_id) {
    $form->{vendor_id} = $vendor_id;
    IR->get_vendor(\%myconfig, \%$form);
  }
  
  my $i = $form->{rowcount};

  if (($form->{"partnumber_$i"} eq "") && ($form->{"description_$i"} eq "")) {

    &check_form;
    
  } else {
   
    IR->retrieve_item(\%myconfig, \%$form);

    my $rows = scalar @{ $form->{item_list} };
    
    if ($rows) {
      $form->{"qty_$i"}                     = 1 unless ($form->{"qty_$i"});
      
      if ($rows > 1) {
	&select_item;
	exit;
      } else {
	map { $form->{item_list}[$i]{$_} =~ s/"/&quot;/g } qw(partnumber description unit);
	map { $form->{"${_}_$i"} = $form->{item_list}[0]{$_} } keys %{ $form->{item_list}[0] };
	$form->{"sellprice_$i"} = $form->format_amount(\%myconfig, $form->{"sellprice_$i"});
	$form->{"qty_$i"} =  $form->format_amount(\%myconfig, $form->{"qty_$i"}, "", "nat");
      }

      &display_form;

    } else {
      # ok, so this is a new part
      # ask if it is a part or service item

      $form->{"id_$i"}		= 0;
      $form->{"unit_$i"}	= $locale->text('ea');

      &new_item;
      
    }
  }
}



sub select_item {
  
  # list items with radio button on a form
  $form->header;

  print qq|

<h2>|.$locale->text('Select from one of the items below').qq|</h2>

<form method=post action=$form->{script}>

<table border=1>
<tr><td>

<table border=0>

<tr bgcolor=336666>
  <th>&nbsp;</th>
  <th><font color=ffffff>|.$locale->text('Number').qq|</th>
  <th><font color=ffffff>|.$locale->text('Description').qq|</th>
  <th><font color=ffffff>|.$locale->text('Price').qq|</th>
|;

  my $i = 0;
  foreach $ref (@{ $form->{item_list} }) {
    $checked = ($i++) ? "" : "checked";
    
    map { $ref->{$_} =~ s/"/&quot;/g } qw(partnumber description unit);
    
    $ref->{sellprice} = $form->format_amount(\%myconfig, $ref->{sellprice});
    
    print qq|
<tr>
  <td><input name=ndx class=radio type=radio value=$i $checked></td>
  <td><input name="new_partnumber_$i" type=hidden value="$ref->{partnumber}">$ref->{partnumber}</td>
  <td><input name="new_description_$i" type=hidden value="$ref->{description}">$ref->{description}</td>
  <td align=right><input name="new_sellprice_$i" type=hidden value=$ref->{sellprice}>$ref->{sellprice}</td>
</tr>
<input name="new_inventory_accno_$i" type=hidden value=$ref->{inventory_accno}>
<input name="new_income_accno_$i" type=hidden value=$ref->{income_accno}>
<input name="new_expense_accno_$i" type=hidden value=$ref->{expense_accno}>
<input name="new_unit_$i" type=hidden value="$ref->{unit}">
<input name="new_taxaccounts_$i" type=hidden value="$ref->{taxaccounts}">

<input name="new_id_$i" type=hidden value=$ref->{id}>
|;
  }

print qq|
</table>
</td></tr>
</table>

|;

# delete action variable and item_list hash
delete $form->{action};
delete $form->{item_list};

# save all other form variables
foreach $key (keys %${form}) {
  $form->{$key} =~ s/"/&quot;/g;
  print qq|<input name=$key type=hidden value="$form->{$key}">\n|;
}

print qq|
<input type=hidden name=nextsub value=item_selected>

<input class=submit type=submit name=action value="|.$locale->text('Continue').qq|">
</form>
|;

}



sub new_item {
  
  $form->isblank("partnumber_$form->{rowcount}", $locale->text('Part Number missing!'));

  # ask if it is a service item or part
  $form->header;

  print qq|

<h2>|.$locale->text('Number').qq|: $form->{"partnumber_$form->{rowcount}"}

<p><font color=red>|.$locale->text('Item not on file!').qq|</font>

<p>
|.$locale->text('What type of item is this?').qq|</h2>

<form method=post action=$form->{script}>

<p>
<input class=radio type=radio name=itemtype value=part checked> |.$locale->text('Part').qq|
<br><input class=radio type=radio name=itemtype value=service> |.$locale->text('Service').qq|

|;

# delete action variable
delete $form->{action};

# save all other form variables
foreach $key (keys %${form}) {
  $form->{$key} =~ s/"/&quot;/g;
  print qq|<input name=$key type=hidden value="$form->{$key}">\n|;
}

print qq|
<input type=hidden name=nextsub value=part_or_service>

<p>
<input class=submit type=submit name=action value="|.$locale->text('Continue').qq|">
</form>
|;

}



sub item_selected {
  
  # replace the last row with the checked row
  $i = $form->{rowcount};

  # index for new item
  $j = $form->{ndx};
  
  map { $form->{"${_}_$i"} = $form->{"new_${_}_$j"} } qw(id partnumber description sellprice inventory_accno income_accno expense_accno unit taxaccounts);

  &display_form;
  
}



sub part_or_service {

  $i = $form->{rowcount};
  
  if ($form->{itemtype} eq "part") {
    $form->{"inventory_accno_$i"} = $form->{inventory_accno};
  }

  $form->{"taxaccounts_$i"} = $form->{"tax$form->{itemtype}"};
  $form->{"income_accno_$i"} = $form->{income_accno};
  $form->{"expense_accno_$i"} = $form->{expense_accno};
  
  &display_form;

}



sub check_form {
  
  my @a = ();
  my $count = 0;
  my @flds = qw(id partnumber description qty sellprice unit inventory_accno income_accno expense_accno taxaccounts);

  # remove any empty numbers
  for my $i (1 .. ($form->{rowcount} - 1)) {
    if ($form->{"partnumber_$i"}) {
      push @a, {};
      my $j = $#a;

      map { $a[$j]->{$_} = $form->{"${_}_$i"} } @flds;
      $count++;
    }
  }


  for $i (1 .. $count) {
    my $j = $i - 1;
    map { $form->{"${_}_$i"} = $a[$j]->{$_} } @flds;
  }


  for $i ($count + 1 .. $form->{rowcount}) {
    map { delete $form->{"${_}_$i"} } @flds;
  }

  $form->{rowcount} = $count;

  &display_form;

}



sub post_invoice {

  $form->isblank("invnumber", $locale->text('Invoice Number missing!'));
  $form->isblank("invdate", $locale->text('Invoice Date missing!'));

  
  # check if items are valid
  for $i (1 .. $form->{rowcount}) {
    if (!($form->{"income_accno_$i"}) && $form->{"qty_$i"}) {
      $form->isblank("income_accno_$i", $locale->text('Invalid Item in Row') . " $i");
    }
  }

    # if the vendor changed get new values
  ($null, $vendor_id) = split /--/, $form->{vendor};
  if ($form->{vendor_id} ne $vendor_id) {
    $form->{vendor_id} = $vendor_id;
    IR->get_vendor(\%myconfig, \%$form);
  }
 
  
  ($form->{AP}) = split /--/, $form->{AP};
  ($form->{AP_paid}) = split /--/, $form->{AP_paid};
  
  
  if (IR->post_invoice(\%myconfig, \%$form)) {
    if ($form->{new_invoice}) {
    
      $form->header;
      
      print qq|<form method=post action=$form->{script}>\n|;

      print qq|
<h2><font color=red>|.$locale->text('Confirm!').qq|</font></h2>

<h4>|.$locale->text('Do you want to create another Invoice').qq|</h4>
<p>
<input name=action class=submit type=submit value="|.$locale->text('Yes').qq|">
<input name=action class=submit type=submit value="|.$locale->text('No').qq|">

<input name=callback type=hidden value="$form->{callback}">

<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

</form>
|;

    } else {
      $form->isposted(1);
    }
  }

}


sub yes {

  $form->isposted(1);

}


sub no {

  $form->{callback} = qq|login.pl?action=company_logo&login=$form->{login}&path=$form->{path}|;
  $form->redirect;

}


sub edit {

  $form->{title} = "Edit Vendor Invoice";

  IR->link_invoice(\%myconfig, \%$form);

  &create_links;

  &prepare_invoice;
  
}


  
sub delete_invoice {

  $form->header;

  print qq|<form method=post action=$form->{script}>\n|;

  foreach $key (keys %$form) {
    $form->{$key} =~ s/"/&quot;/g;
    print qq|<input type=hidden name=$key value="$form->{$key}">\n|;
  }

  print qq|
<h2><font color=red>|.$locale->text('Confirm!').qq|</font></h2>

<h4>|.$locale->text('Are you sure you want to delete Invoice Number').qq| $form->{invnumber}</h4>
<p>
<input name=action class=submit type=submit value="|.$locale->text('Yes Delete Invoice').qq|">
</form>
|;

}



sub yes_delete_invoice {

  $form->isdeleted(IR->delete_invoice(\%myconfig, \%$form));

}


