#=====================================================================
# SQL-Ledger Accounting
# Copyright (c) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors: Antonio Gallardo <agssa@ibw.com.ni>
#                Benjamin Lee <benjaminlee@consultant.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
#
# module for preparing Income Statement and Balance Sheet
# 
# CHANGE LOG:
#   DS. 2000-07-04  Created
#   AG. 2001-01-20  Added trial balance
#                   Added totals on reports
#   DS. 2001-02-16  included links on trial balance
#   BL. 2001-05-31  moved retained earnings account in balance sheet
#   BL. 2001-06-17  aging reports
#   DS. 2001-07-09  added headings to trial balance
#
#======================================================================


use DBI;
use SL::Form;
use SL::RP;


$form = new Form;


if ($form->{action}) {
  require("$userspath/$form->{login}.conf");
  $myconfig{dbpasswd} = unpack 'u', $myconfig{dbpasswd};
  map { $form->{$_} = $myconfig{$_} } qw(stylesheet charset);

  $locale = new Locale $myconfig{countrycode}, "rp";
  
  &{ $locale->findsub($form->{action}) };
} else {
  $form->error("action= not defined!");
}

# end of main

# this is for our long dates
# $locale->text('January')
# $locale->text('February')
# $locale->text('March')
# $locale->text('April')
# $locale->text('May')
# $locale->text('June')
# $locale->text('July')
# $locale->text('August')
# $locale->text('September')
# $locale->text('October')
# $locale->text('November')
# $locale->text('December')

# this is for our short month
# $locale->text('Jan')
# $locale->text('Feb')
# $locale->text('Mar')
# $locale->text('Apr')
# $locale->text('May')
# $locale->text('Jun')
# $locale->text('Jul')
# $locale->text('Aug')
# $locale->text('Sep')
# $locale->text('Oct')
# $locale->text('Nov')
# $locale->text('Dec')

# $locale->text('Income Statement')
# $locale->text('Balance Sheet')
# $locale->text('AR Aging')
# $locale->text('AP Aging')


sub report {

  $form->header;
  
  print qq|
<h2>|.$locale->text($form->{title}).qq|</h2>

<form method=post action=$form->{script}>

<table border=1>
<tr valign=top>
<td>
<table>

|;

  if ($form->{title} eq "Income Statement") {
    print qq|
<tr>
  <th align=right>|.$locale->text('From').qq| ($myconfig{dateformat})</th><td><input name=fromdate size=10></td>
  <th align=right>|.$locale->text('To').qq|</th><td><input name=todate size=10></td>
</tr>

<tr>
  <th align=left>|.$locale->text('Compare to').qq|</th>
</tr>

<tr>
  <th align=right>|.$locale->text('From').qq| ($myconfig{dateformat})</th><td><input name=comparefromdate size=10></td>
  <th align=right>|.$locale->text('To').qq|</th><td><input name=comparetodate size=10></td>
</tr>
|;

  }


  if ($form->{title} eq "Balance Sheet") {
    print qq|
<tr>
  <th align=right>|.$locale->text('as of').qq| ($myconfig{dateformat})</th><td><input name=asofdate size=10></td>
</tr>

<tr>
  <th align=left>|.$locale->text('Compare to').qq|</th>
</tr>

<tr>
  <th align=right>|.$locale->text('as of').qq| ($myconfig{dateformat})</th><td><input name=compareasofdate size=10></td>
</tr>
|;
  }


  if ($form->{title} eq "Trial Balance") {
    print qq|
<tr>
  <th align=right>|.$locale->text('From').qq| ($myconfig{dateformat})</th><td><input name=fromdate size=10></td>
  <th align=right>|.$locale->text('To').qq|</th><td><input name=todate size=10></td>
</tr>

<tr>
  <th>|.$locale->text('Include in Report').qq|</th>
  <td><input name=l_heading class=checkbox type=checkbox value=Y> |.$locale->text('Heading').qq|
<input name=l_subtotal class=checkbox type=checkbox value=Y> |.$locale->text('Subtotal').qq|
  </td>
</tr>
|;
  }


  if ($form->{title} eq "AP Aging") {
    # setup vendor selection
    $form->all_vc(\%myconfig, "vendor");

    $selection = "<option selected>" . $locale->text('All Vendors') ."\n";
    
    foreach $ref (@{ $form->{all_vendor} }) {
      $selection .= "<option>$ref->{name}--$ref->{id}\n";
    }
    
    print qq|
<tr>
  <th align=right>|.$locale->text('Vendor').qq|</th><td colspan=3><select name=vendor>$selection</select></td>
</tr>

<tr>
  <th align=right>|.$locale->text('To').qq| ($myconfig{dateformat})</th><td><input name=todate size=10></td>
</tr>
|;
  }


  if ($form->{title} eq "AR Aging") {
    # setup customer selection
    $form->all_vc(\%myconfig, "customer");

    $selection = "<option selected>" . $locale->text('All Customers') ."\n";
    
    foreach $ref (@{ $form->{all_customer} }) {
      $selection .= "<option>$ref->{name}--$ref->{id}\n";
    }
    
    print qq|
<tr>
  <th align=right>|.$locale->text('Customer').qq|</th><td colspan=3><select name=customer>$selection</select></td>
</tr>

<tr>
  <th align=right>|.$locale->text('To').qq| ($myconfig{dateformat})</th><td><input name=todate size=10></td>
</tr>
|;
  }


  print qq|
</table>

</td>
</tr>
</table>

<p>
<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

<input class=submit type=submit name=action value="|.$locale->text('Generate ' . $form->{title}).qq|">
</form>

<p>|.$locale->text("To show current $form->{title}, leave fields blank.");

}


# these are buttons as well, so flag them to generate a sub entry
# class=submit type=submit $locale->text('Generate Income Statement')
# class=submit type=submit $locale->text('Generate Balance Sheet')
# class=submit type=submit $locale->text('Generate Trial Balance')
# class=submit type=submit $locale->text('Generate AR Aging')
# class=submit type=submit $locale->text('Generate AP Aging')

# $locale->text("To show current Income Statement, leave fields blank.")
# $locale->text("To show current Balance Sheet, leave fields blank.")
# $locale->text("To show current Trial Balance, leave fields blank.")
# $locale->text("To show current AR Aging, leave fields blank.")
# $locale->text("To show current AP Aging, leave fields blank.")


sub generate_income_statement {

  RP->income_statement(\%myconfig, \%$form);


  # if there are any dates construct a where
  if ($form->{fromdate} || $form->{todate}) {
    $form->{todate} = $form->current_date(\%myconfig) unless ($form->{todate});

    $longtodate = $locale->date(\%myconfig, $form->{todate}, 1);
    $shorttodate = $locale->date(\%myconfig, $form->{todate});
    
    $longfromdate = $locale->date(\%myconfig, $form->{fromdate}, 1);
    $shortfromdate = $locale->date(\%myconfig, $form->{fromdate});
    
    $form->{this_period} = "$shortfromdate<br>$shorttodate";
    $form->{period} = $locale->text('for Period')."<br>$longfromdate ".$locale->text('To')." $longtodate";
  } else {
    $form->{period} = $locale->date(\%myconfig, $form->current_date(\%myconfig), 1);
  }


  if ($form->{comparefromdate} || $form->{comparetodate}) {
    $longcomparefromdate = $locale->date(\%myconfig, $form->{comparefromdate}, 1);
    $shortcomparefromdate = $locale->date(\%myconfig, $form->{comparefromdate});
    
    $longcomparetodate = $locale->date(\%myconfig, $form->{comparetodate}, 1);
    $shortcomparetodate = $locale->date(\%myconfig, $form->{comparetodate});
    
    $form->{last_period} = "$shortcomparefromdate<br>$shortcomparetodate";
    $form->{period} .= "<br>$longcomparefromdate ".$locale->text('To')." $longcomparetodate";
  }

 
  # setup variables for the form
  map { $form->{$_} = $myconfig{$_} } (qw(company address businessnumber));
  $form->{address} =~ s/\\n/<br>/g;

  $form->{templates} = $myconfig{templates};

  $form->{IN} = "income_statement.html";
  
  $form->header;
  $form->parse_template;

}


sub generate_balance_sheet {


  RP->balance_sheet(\%myconfig, \%$form);

  
  $form->{period} = $locale->date(\%myconfig, $form->current_date(\%myconfig), 1);
  
  # define Retained Earnings account
  push(@{$form->{equity_account}}, $locale->text('Retained Earnings'));

  $form->{this_period} = $locale->date(\%myconfig, $form->{asofdate});
  $form->{last_period} = $locale->date(\%myconfig, $form->{compareasofdate});

  $form->{IN} = "balance_sheet.html";

  # setup company variables for the form
  map { $form->{$_} = $myconfig{$_} } (qw(company address businessnumber nativecurr));
  $form->{address} =~ s/\\n/<br>/g;

  $form->{templates} = $myconfig{templates};
	  
  $form->header;
  $form->parse_template;

}


# Antonio Gallardo
#
# D.S. Feb 16, 2001
# included links to display transactions for period entered
# added headers and subtotals
#
sub generate_trial_balance {


  # get for each account initial balance, debits and credits
  RP->trial_balance_details(\%myconfig, \%$form);


  # if there are any dates
  if ($form->{fromdate} || $form->{todate}) {
    $form->{todate} = $form->current_date(\%myconfig) unless ($form->{todate});
    
    $fromdate = $locale->date(\%myconfig, $form->{fromdate}, 1);
    $todate = $locale->date(\%myconfig, $form->{todate}, 1);
    
    $form->{period} = qq|$fromdate |.$locale->text('To').qq| $todate|;
  } else {
    $form->{period} = $locale->date(\%myconfig, $form->current_date(\%myconfig), 1);
  }


  $form->header;


  print qq|<b><font size=+4>|.$locale->text('Trial Balance').qq|
  </font><br>$form->{period}</b>

<p>
<table border=1>
<tr bgcolor=000000>
<th><font color=ffffff>|.$locale->text('Account').qq|</th>
<th><font color=ffffff>|.$locale->text('Description').qq|</th>
<th><font color=ffffff>|.$locale->text('Debit').qq|</th>
<th><font color=ffffff>|.$locale->text('Credit').qq|</th>
</tr>
|;

  # reverse scan TB and set empty headers not to display
  $heading = 1;
  foreach $ref (reverse @{ $form->{TB} }) {
    $ref->{display} = 1;
    if ($ref->{charttype} eq "H") {
      $ref->{display} = 0 if ($heading);
      $heading = 1;
    } else {
      $heading = 0;
      unless ($form->{lastaccno}{$ref->{category}} != 0) {
	$form->{lastaccno}{$ref->{category}} = $ref->{accno} + 0.5;
      }
    }
    # subtotal sections
    $form->{subtotal}{$ref->{category}}{debit} += $ref->{debit};
    $form->{subtotal}{$ref->{category}}{credit} += $ref->{credit};
  }
  
  foreach $key (keys %{ $form->{lastaccno} }) {
    push @{ $form->{TB} }, { charttype => 'S',
                             accno => $form->{lastaccno}{$key},
			     debit => $form->{subtotal}{$key}{debit},
			     credit => $form->{subtotal}{$key}{credit},
			     display => 1 };
  }
  
  # sort the whole thing by account numbers and display
  foreach $ref (sort { $a->{accno} <=> $b->{accno} } @{ $form->{TB} }) {
   
    $href = "ca.pl?path=$form->{path}&action=list_transactions&accno=$ref->{accno}&login=$form->{login}&fromdate=$form->{fromdate}&todate=$form->{todate}&sort=transdate&l_subtotal=$form->{l_subtotal}&description=" . $form->escape($ref->{description});

    $debit = $form->format_amount(\%myconfig, $ref->{debit});
    $credit = $form->format_amount(\%myconfig, $ref->{credit});

      
    if ($ref->{charttype} eq "H") {
      next unless ($form->{l_heading});
      
      $column_data{accno} = "<th>&nbsp;</th>";
      $column_data{description} = "<th><font color=ffffff>$ref->{description}</th>";
      $column_data{debit} = "<th>&nbsp;</th>";
      $column_data{credit} = "<th>&nbsp;</th>";
    }
    
    if ($ref->{charttype} eq "A") {
      $column_data{accno} = "<td><a href=$href>$ref->{accno}</a></td>";
      $column_data{description} = "<td>$ref->{description}</td>";
      $column_data{debit} = "<td align=right>&nbsp;$debit</td>";
      $column_data{credit} = "<td align=right>&nbsp;$credit</td>";
    
      $total_debit += $ref->{debit};
      $total_credit += $ref->{credit};
    }
    
    # subtotal
    if ($ref->{charttype} eq "S") {
      next unless ($form->{l_subtotal});

      map { $column_data{$_} = "<th>&nbsp;</th>" } qw(accno description debit credit);
      
      $column_data{debit} = "<th align=right><font color=ffffff>&nbsp;".$form->format_amount(\%myconfig, $ref->{debit})."</th>";
      $column_data{credit} = "<th align=right><font color=ffffff>&nbsp;".$form->format_amount(\%myconfig, $ref->{credit})."</th>";
    }
    
    if ($ref->{display}) {
      if ($ref->{charttype} eq "H") {
	print qq|<tr bgcolor=336666>\n|;
      }
      if ($ref->{charttype} eq "S") {
	print qq|<tr bgcolor=333366>\n|;
      }
      if ($ref->{charttype} eq "A") {
	print qq|<tr>\n|;
      }
      
      map { print "$column_data{$_}\n" } (qw(accno description debit credit));
      
      print qq|</tr>|;
    }
  }

  $total_debit = $form->format_amount(\%myconfig, $total_debit);
  $total_credit = $form->format_amount(\%myconfig, $total_credit);

  print qq|<tr bgcolor=000000>
           <th>&nbsp;</th>
           <th>&nbsp;</th>
           <th align=right><font color=ffffff>&nbsp;$total_debit</th>
           <th align=right><font color=ffffff>&nbsp;$total_credit</th>
	   </tr>
	   </table>
<p>
|;

  print $locale->text('Click on account number to display transactions');

}


sub generate_ar_aging {
  
  # split customer
  ($customer, $form->{customer_id}) = split(/--/, $form->{customer});

  $form->{ct} = "customer";
  $form->{arap} = "ar";

  $form->{title} = $locale->text('Accounts Receivables Aging Report');

  &aging;
  
}


sub generate_ap_aging {
  
  # split vendor
  ($vendor, $form->{vendor_id}) = split(/--/, $form->{vendor});

  $form->{ct} = "vendor";
  $form->{arap} = "ap";
  
  $form->{title} = $locale->text('Accounts Payables Aging Report');

  &aging;
  
}


sub aging {

  RP->aging(\%myconfig, \%$form);

  $form->header;

  $column_header{ct} = qq|<th><font color=ffffff>|.$locale->text(ucfirst $form->{ct}).qq|</th>|;
  $column_header{invnumber} = qq|<th><font color=ffffff>|.$locale->text('Invoice Number').qq|</th>|;
  $column_header{transdate} = qq|<th><font color=ffffff>|.$locale->text('Invoice Date').qq|</th>|;
  $column_header{duedate} = qq|<th><font color=ffffff>|.$locale->text('Due Date').qq|</th>|;
  $column_header{c0} = qq|<th><font color=ffffff>0 - 30</th>|;
  $column_header{c30} = qq|<th><font color=ffffff>31 - 60</th>|;
  $column_header{c60} = qq|<th><font color=ffffff>61 - 90</th>|;
  $column_header{c90} = qq|<th><font color=ffffff>90+</th>|;
  
  @column_index = (qw(ct invnumber transdate duedate c0 c30 c60 c90));

  
  if ($form->{arap} eq 'ar') {
    if ($form->{customer_id}) {
      $form->{title} .= "<br>$customer";
      shift @column_index;
    }
  }
  if ($form->{arap} eq 'ap') {
    if ($form->{vendor_id}) {
      $form->{title} .= "<br>$vendor";
      shift @column_index;
    }
  }
  
  $todate = $locale->date(\%myconfig, $form->{todate}, 1);
  $form->{title} .= "<br>".$locale->text('for Period')." ".$locale->text('To')." $todate";

  print qq|
  <h1>$form->{title}</h1>
  
<table border=1>
<tr bgcolor=336666>
|;

  map { print "$column_header{$_}\n" } @column_index;
  
  print qq|
</tr>|;


  $ctid = 0;
  $subtotal = 0;

  foreach $ref (@{ $form->{AG} }) {

    if ($ctid != $ref->{id}) {

      if ($subtotal) {
	$c0subtotal = $form->format_amount(\%myconfig, $c0subtotal);
	$c30subtotal = $form->format_amount(\%myconfig, $c30subtotal);
	$c60subtotal = $form->format_amount(\%myconfig, $c60subtotal);
	$c90subtotal = $form->format_amount(\%myconfig, $c90subtotal);
      }
      
      $column_data{ct} = qq|<th>&nbsp;</th>|;
      $column_data{invnumber} = qq|<th>&nbsp;</th>|;
      $column_data{transdate} = qq|<th>&nbsp;</th>|;
      $column_data{duedate} = qq|<th>&nbsp;</th>|;
      $column_data{c0} = qq|<th align=right><font color=ffffff>&nbsp;$c0subtotal</font></th>|;
      $column_data{c30} = qq|<th align=right><font color=ffffff>&nbsp;$c30subtotal</font></th>|;
      $column_data{c60} = qq|<th align=right><font color=ffffff>&nbsp;$c60subtotal</font></th>|;
      $column_data{c90} = qq|<th align=right><font color=ffffff>&nbsp;$c90subtotal</font></th>|;
     
      if ($subtotal) {
      
	# print totals
	print qq|<tr bgcolor=333366>|;

	map { print "$column_data{$_}\n" } @column_index;

	print qq|
      </tr>|;
      }
   
      $subtotal = 1;

      $c0subtotal = 0;
      $c30subtotal = 0;
      $c60subtotal = 0;
      $c90subtotal = 0;

      $column_data{ct} = qq|<td>$ref->{name}</td>|;
    } else {
      $column_data{ct} = qq|<td>&nbsp;</td>|;
    }

    $c0subtotal += $ref->{c0};
    $c30subtotal += $ref->{c30};
    $c60subtotal += $ref->{c60};
    $c90subtotal += $ref->{c90};
    
    $ref->{c0} = $form->format_amount(\%myconfig, $ref->{c0});
    $ref->{c30} = $form->format_amount(\%myconfig, $ref->{c30});
    $ref->{c60} = $form->format_amount(\%myconfig, $ref->{c60});
    $ref->{c90} = $form->format_amount(\%myconfig, $ref->{c90});
    
    $column_data{invnumber} = qq|<td>$ref->{invnumber}</td>|;
    $column_data{transdate} = qq|<td>$ref->{transdate}</td>|;
    $column_data{duedate} = qq|<td>$ref->{duedate}</td>|;
    $column_data{c0} = qq|<td align=right>&nbsp;$ref->{c0}</td>|;
    $column_data{c30} = qq|<td align=right>&nbsp;$ref->{c30}</td>|;
    $column_data{c60} = qq|<td align=right>&nbsp;$ref->{c60}</td>|;
    $column_data{c90} = qq|<td align=right>&nbsp;$ref->{c90}</td>|;
    

    print qq|<tr>|;

    map { print "$column_data{$_}\n" } @column_index;

    print qq|
</tr>|;
 
    $ctid = $ref->{id};

  }
  
  # print totals
  $c0subtotal = $form->format_amount(\%myconfig, $c0subtotal);
  $c30subtotal = $form->format_amount(\%myconfig, $c30subtotal);
  $c60subtotal = $form->format_amount(\%myconfig, $c60subtotal);
  $c90subtotal = $form->format_amount(\%myconfig, $c90subtotal);
  
  print qq|<tr bgcolor=333366>|;

  $column_data{ct} = qq|<th>&nbsp;</th>|;
  $column_data{invnumber} = qq|<th>&nbsp;</th>|;
  $column_data{transdate} = qq|<th>&nbsp;</th>|;
  $column_data{duedate} = qq|<th>&nbsp;</th>|;
  $column_data{c0} = qq|<th align=right><font color=ffffff>&nbsp;$c0subtotal</font></th>|;
  $column_data{c30} = qq|<th align=right><font color=ffffff>&nbsp;$c30subtotal</font></th>|;
  $column_data{c60} = qq|<th align=right><font color=ffffff>&nbsp;$c60subtotal</font></th>|;
  $column_data{c90} = qq|<th align=right><font color=ffffff>&nbsp;$c90subtotal</font></th>|;

  map { print "$column_data{$_}\n" } @column_index;
  
  print qq|
</table>|;

}


