#=====================================================================
# SQL-Ledger Accounting
# Copyright (c) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
#
# setup module
# add/edit/delete users
#
# CHANGE LOG:
#   DS. 2000-07-04  Created
#   DS. 2001-06-04  Added selection menu for language
#                   changed chart to load from database
#                   instead of external file
#   DS. 2001-07-22  added database driver selection
#
#======================================================================


use DBI;
use SL::Form;
use SL::User;


$form = new Form;

# set sid for oracle
$form->{sid} = $sid;


if ($form->{action}) {

  &{ $form->{action} };

} else {

  # if there are no drivers bail out
  $form->error("No database drivers available") unless (User->dbdrivers);

  unless (-f $memberfile) {
    open(FH, ">$memberfile") || $form->error("$!: $memberfile");
    print FH qq|# SQL-Ledger Accounting members

[root login]
password=

|;
    close FH;
  }

  &adminlogin;

}

# end


sub adminlogin {

  print qq|Content-type: text/html

<head>
  <title>SQL-Ledger Accounting, Version $version - Administration</title>
</head>

<body bgcolor="#FBFFE7">

<center>

<a href="http://www.sql-ledger.org"><img src=sql-ledger.gif border=0></a>
<h1>
Version $version
<p>Administration
</h1>

<form method=post action=$form->{script}>

<table>
<tr>
<td>

<table>

<tr>
<th align=right>Password</th><td><input type=password name=rpw></td>
</tr>
</table>
</td>

<td>
<input class=submit type=submit name=action value="Login">
</td>
</tr>

<input type=hidden name=root value="root login">
<input type=hidden name=path value=$form->{path}>

</table>

</form>

For a new version click here
<a href=http://www.sql-ledger.org>SQL-Ledger website</a>

|;

}




sub login {

  &check_password;

  &list_users;

}



sub add_user {
  
  $form->{title} = "SQL-Ledger Accounting Administration / Add User";

  &form_header;
  &form_footer;
  
}



sub edit {

  $form->{title} = "SQL-Ledger Accounting Administration / Edit User";
  $form->{edit} = 1;

  &form_header;
  &form_footer;

}


sub form_footer {

  if ($form->{edit}) {
    $delete = qq|<input class=submit type=submit name=action value="Delete">
<input type=hidden name=edit value=1>|;
  }

  print qq|

<input name=callback type=hidden value="$form->{script}?action=list_users&path=$form->{path}&root=$form->{root}&rpw=$form->{rpw}">
<input type=hidden name=root value="$form->{root}">
<input type=hidden name=path value=$form->{path}>
<input type=hidden name=rpw value=$form->{rpw}>

<tr><td>
<input class=submit type=submit name=action value="Save">
$delete
</td></tr>

</table>

</form>

|;

}


sub list_users {

  &check_password;
  
  open(FH, "$memberfile") || $form->error("$!: $memberfile");
  
  while (<FH>) {
    chop;
    
    if (/^\[.*\]/) {
      $login = $_;
      chop $login;
      $login = substr($login,1);
    }

    if (/^(name=|company=|dbname=|templates=)/) {
      chop ($var = $&);
      ($null, $member{$login}{$var}) = split /=/, $_, 2;
    }
  }
  
  close(FH);

  foreach $item (User->dbdrivers) {
    $dbdrivers .= qq|<br><input name=action class=submit type=submit value="$item Database Administration">|;
  }

    
  $form->{title} = qq|SQL-Ledger Accounting Administration|;
  
  print qq|Content-type: text/html

<head>
  <title>$form->{title}</title>
</head>

<body bgcolor="#FBFFE7">

<center>
<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<table><tr><td>
<table border=1>

<tr bgcolor=336666>
  <th><font color=ffffff>login</th>
  <th><font color=ffffff>Name</th>
  <th><font color=ffffff>Company</th>
  <th><font color=ffffff>Database</th>
  <th><font color=ffffff>Templates</th>
</tr>

|;

foreach $key (sort keys %member) {
  $href = "$script?action=edit&login=$key&path=$form->{path}&root=$form->{root}&rpw=$form->{rpw}";
  $href =~ s/ /%20/g;
  
  $member{$key}{templates} =~ s/$templates\///;

  print qq|
<tr>
  <td><a href=$href>$key</a></td>
  <td>$member{$key}{name}</td>
  <td>$member{$key}{company}</td>
  <td>$member{$key}{dbname}</td>
  <td>$member{$key}{templates}</td>
</tr>|;
}

print qq|
</table>

</td></tr>

<tr><td>
<input type=hidden name=path value=$form->{path}>
<input type=hidden name=rpw value=$form->{rpw}>
<input type=hidden name=root value="$form->{root}">

<input class=submit type=submit name=action value="Add User">
<input class=submit type=submit name=action value="Change Admin Password">

$dbdrivers

</td></tr>
</table>

</form>

</center>

Click on login name to edit!
<br>To add a user to a group (company) edit a name,
change the login name and save.
A new user with the same variables will then be saved under
the new login name.

</body>
</html>
|;

}




sub form_header {

 
  $myconfig->{dbport} = $form->{dbport};
  $myconfig->{dbhost} = $form->{dbhost};
  
  # if there is a login, get user
  if ($form->{login}) {
    # get user
    $myconfig = new User "$memberfile", "$form->{login}";

    $myconfig->{signature} =~ s/\\n/\n/g;
    $myconfig->{address} =~ s/\\n/\n/g;

    # strip basedir from templates directory
    $myconfig->{templates} =~ s/$templates\///;

    $myconfig->{dbpasswd} = unpack 'u', $myconfig->{dbpasswd};

    map { $form->{$_} = $myconfig->{$_} } (qw(dbdriver dbhost dbport dbuser dbname));

    $form->{dbdefault} = $myconfig->{dbname};
  }


  foreach $item (qw(yy-mm-dd mm-dd-yy mm/dd/yy dd-mm-yy dd/mm/yy dd.mm.yy)) {
    $dateformat .= ($item eq $myconfig->{dateformat}) ? "<option selected>$item\n" : "<option>$item\n";
  }

  foreach $item (qw(1,000.00 1000.00 1.000,00 1000,00)) {
    $numberformat .= ($item eq $myconfig->{numberformat}) ? "<option selected>$item\n" : "<option>$item\n";
  }


  %countrycodes = (split /:/, $countrycodes);
  $countrycodes = "";
  foreach $key (sort keys %countrycodes) {
    $countrycodes .= ($key eq $myconfig->{countrycode}) ? "<option selected>$key--$countrycodes{$key}\n" : "<option>$key--$countrycodes{$key}\n";
  }


  # is there a templates basedir
  unless (-d "$templates") {
    $form->error("Directory: $templates does not exist");
  }

  opendir TEMPLATEDIR, "$templates/." || $form-error("$!: $templates");
  @all = grep !/^\.\.?$/, readdir TEMPLATEDIR;
  closedir TEMPLATEDIR;

  @allhtml = sort grep /\.html/, @all;
  @alldir = grep !/\.html/, @all;
  
  @allhtml = reverse grep !/Default/, @allhtml;
  push @allhtml, 'Default';
  @allhtml = reverse @allhtml;
  
  foreach $item (@alldir) {
    if ($item eq $myconfig->{templates}) {
      $usetemplates .= qq|<option selected>$item\n|;
    } else {
      $usetemplates .= qq|<option>$item\n|;
    }
  }
  
  $lastitem = $allhtml[0];
  $lastitem =~ s/_.*//g;
  $mastertemplates = qq|<option>$lastitem\n|;
  foreach $item (@allhtml) {
    $item =~ s/_.*//g;
    
    if ($item ne $lastitem) {
      $mastertemplates .= qq|<option>$item\n|;
      $lastitem = $item;
    }
  }


  foreach $item (User->dbdrivers) {
    if ($item eq $myconfig->{dbdriver}) {
      $dbdrivers .= "<option selected>$item\n";
    } else {
      $dbdrivers .= "<option>$item\n";
    }
  }
  
  
  print qq|Content-type: text/html

<head>
  <title>$form->{title}</title>
</head>

<body bgcolor="#FBFFE7">

<form method=post action=$form->{script}>

<center>
<table>

<tr><th bgcolor=336666><font size=+1 color=ffffff>$form->{title}</font></th></tr>

<tr><td>

<table border=1><tr><td>

<table>
<tr>
  <th align=right>Login</th>
  <td><input name=login value="$myconfig->{login}">
</tr>

<tr>
  <th align=right>Password</th>
  <td><input type=password name=password size=8 value=$myconfig->{password}></td>
  <input type=hidden name=old_password value=$myconfig->{password}>
</tr>

<tr>
  <th align=right>Name</th>
  <td><input name=name size=15 value="$myconfig->{name}"></td>
</tr>

<tr>
  <th align=right>E-mail</th>
  <td><input name=email size=30 value=$myconfig->{email}></td>
</tr>

<tr valign=top>
  <th align=right>Signature</th>
  <td><textarea name=signature rows=3 cols=30>$myconfig->{signature}</textarea></td>
</tr>

<tr>
  <th align=right>Phone</th>
  <td><input name=tel size=14 value="$myconfig->{tel}">
  <b>Fax</b>
  <input name=fax size=14 value="$myconfig->{fax}"></td>
</tr>

<tr>
  <th align=right>Company</th>
  <td><input name=company size=30 value="$myconfig->{company}"></td>
</tr>

<tr valign=top>
  <th align=right>Address</th>
  <td><textarea name=address rows=3 cols=30>$myconfig->{address}</textarea></td>
</tr>

<tr>
  <th align=right>Date Format</th>
  <td><select name=dateformat>$dateformat</select>
  </td>
</tr>

<tr>
  <th align=right>Number Format</th>
  <td><select name=numberformat>$numberformat</select>
  </td>
</tr>

<tr>
  <th align=right>Language</th>
  <td>
  <select name=countrycode>$countrycodes</select>
  </td>
</tr>

<tr>
  <th align=right>Charset</th>
  <td>
  <input name=charset value="$myconfig->{charset}">
  </td>
</tr>

<tr>
  <th align=right>Stylesheet</th>
  <td>
  <input name=stylesheet value="$myconfig->{stylesheet}">
  </td>
</tr>

<tr>
  <th align=right>Printer</th>
  <td><input name=printer size=20 value="$myconfig->{printer}"></td>
</tr>

<tr>
  <th align=right>Shipping Point</th>
  <td><input name=shippingpoint size=15 value="$myconfig->{shippingpoint}">
</tr>

<tr>
  <th align=right>Use Templates</th>
  <td><select name=usetemplates>$usetemplates</select></td>
</tr>

<tr>
  <th align=right>New Templates</th>
  <td><input name=newtemplates></td>
</tr>

<tr>
  <th align=right>Setup Templates</th>
  <td><select name=mastertemplates>$mastertemplates</select></td>
</tr>

<tr bgcolor=336666>
  <th colspan=2><font color=ffffff>Database</th>
</tr>

<tr>
  <td colspan=2 align=center>

  <table>
  <tr>
    <td>

    <table>
    <tr>
      <th align=right>Driver</th>
      <td><select name=dbdriver>$dbdrivers</select></td>
      <th align=right>Host</th>
      <td><input name=dbhost value=$myconfig->{dbhost}></td>
    </tr>

    <tr>
      <th align=right>Name</th>
      <td><input name=dbname size=10 value=$myconfig->{dbname}></td>
      <th align=right>Port</th>
      <td><input name=dbport size=4 value=$myconfig->{dbport}></td>
    </tr>

    <tr>
      <th align=right>User</th>
      <td><input name=dbuser size=10 value=$myconfig->{dbuser}></td>
      <th align=right>Password</th>
      <td><input name=dbpasswd size=10 value=$myconfig->{dbpasswd}></td>
    </tr>

    </table>
    </td>
  </tr>

  </table>
  </td>
</tr>

|;


  # access control
  %acs = ( AR => [ 'Add Transaction',
                   'Reports',
		   'Add Customer',
		   'Search Customer',
		   'Customer Invoice',
		   'Receipts',
		   'AR Aging' ],
           AP => [ 'Add Transaction', 
	           'Reports',
		   'Add Vendor',
		   'Search Vendor',
		   'Vendor Invoice',
		   'Payments',
		   'AP Aging' ],
'General Ledger' => [ 'Add Transaction', 'Reports' ],
    Inventory => [ 'Add Part',
                   'Add Service',
		   'Add Assembly',
		   'Reports',
		   'Merchandise Received',
		   'Stock Assembly' ],
      Reports => [ 'Chart of Accounts',
                   'Trial Balance',
		   'Income Statement',
		   'Balance Sheet' ],
        Setup => [ Setup ]
      );


  print qq|
    <tr bgcolor=336666>
      <th colspan=2><font color=ffffff>Access Control</th>
    </tr>
|;


  @acs = split /;/, $myconfig->{acs};
  foreach $item (@acs) {
    ($key, $value) = split /--/, $item;
    $excl{$key}{$value} = 1;
  }
  
  foreach $key (sort keys %acs) {
    print "<tr valign=top><th align=right>$key</th><td>\n";
    foreach $item (@{ $acs{$key} }) {
      $item =~ s/ /_/g;
      $key =~ s/ /_/g;
      $checked = "checked";
      if ($form->{login}) {
	$checked = ($excl{$key}{$item}) ? "" : "checked";
      }

      print qq|<br><input name="${key}--$item" class=checkbox type=checkbox value=1 $checked> $item\n|;
      $menuitems .= "${key}--$item;";
    }
    print qq|</td></tr>\n|;
  }
  
  print qq|<input type=hidden name=acs value="$menuitems">
  
  </td></tr>
 </table>

</td></tr>
</table>

</td></tr>
|;
 
}


sub save {

  &check_password;
  
  # no spaces allowed in login name
  ($form->{login}) = split / /, $form->{login};
  
  
  # check for duplicates
  unless ($form->{edit}) {
    $temp = new User "$memberfile", "$form->{login}";
   
    if ($temp->{login}) {
      $form->error("$form->{login} is already a member!");
    }
  }
  
  # no spaces allowed in directories
  ($form->{newtemplates}) = split / /, $form->{newtemplates};
  
  if ($form->{usetemplates}) {
    $form->{newtemplates} = $form->{usetemplates} unless ($form->{newtemplates});
  } else {
    $form->{newtemplates} = $form->{login};
  }
  
  # fill templates
  if ($form->{newtemplates}) {
    $form->{templates} = $form->{newtemplates};
  }
  
  
  # is there a basedir
  unless (-d "$templates") {
    $form->error("Directory: $templates does not exist");
  }

  # add base directory to $form->{templates}
  $form->{templates} = "$templates/$form->{templates}";


  $myconfig = new User "$memberfile", "$form->{login}";
  
  # redo acs variable and delete all the acs codes
  @acs = split /;/, $form->{acs};

  $form->{acs} = "";
  foreach $item (@acs) {
    unless ($form->{"$item"}) {
      $form->{acs} .= "$item;";
    }
    delete $form->{"$item"};
  }
 

  foreach $item (keys %{$form}) {
    $myconfig->{$item} = $form->{$item};
  }
   
  
  $myconfig->save_member($memberfile, $userspath);

  # create user template directory and copy master files
  unless (-d "$form->{templates}") {
    if (mkdir "$form->{templates}", oct("755")) {
      
      # copy templates to the directory
      opendir TEMPLATEDIR, "$templates/." || $form-error("$!: $templates");
      @templates = grep /$form->{mastertemplates}.*?\.html/, readdir TEMPLATEDIR;
      closedir TEMPLATEDIR;

      foreach $file (@templates) {
	open(TEMP, "$templates/$file") || $form->error("$!: $templates/$file");
	
	$file =~ s/$form->{mastertemplates}_//;
	open(NEW, ">$form->{templates}/$file") || $form->error("$!: $form->{templates}/$file");
	  
	while ($line = <TEMP>) {
	  print NEW $line;
	}
	close(TEMP);
	close(NEW);
      }
    } else {
      $form->error("$!: $form->{templates}");
    }
  }

  $form->redirect;
  
}


sub delete {

  &check_password;
  
  # add base directory to $form->{templates}
  $form->{templates} = ($form->{templates}) ? "$templates/$form->{templates}" : "$templates/$form->{login}";
  

  open(CONF, "$memberfile") || $form->error("$!: $memberfile");
  @config = <CONF>;
  close(CONF);
  
  open(CONF, ">$memberfile") || $form->error("$!: $memberfile");
  
  while ($line = shift @config) {

    if ($line =~ /^\[/) {
      last if ($line =~ /\[$form->{login}\]/);
      $login = &login_name($line);
    }
    
    if ($line =~ /^templates=/) {
      $user{$login} = &get_value($line);
    }

    print CONF $line;
  }

  # remove everything up to next login or EOF
  # and save template variable
  while ($line = shift @config) {
    if ($line =~ /^templates=/) {
      $templatedir = &get_value($line);
    }
    last if ($line =~ /^\[/);
  }

  # this one is either the next login or EOF
  print CONF $line;

  $login = &login_name($line);
  

  while ($line = shift @config) {
    if ($line =~ /^\[/) {
      $login = &login_name($line);
    }
    
    if ($line =~ /^templates=/) {
      $user{$login} = &get_value($line);
    }
    
    print CONF $line;
  }

  close(CONF);

  # scan %user for $templatedir
  foreach $login (keys %user) {
    last if ($found = ($templatedir eq $user{$login}));
  }

  # if found is true keep directory otherwise delete
  unless ($found) {
    # delete it if there is a template directory
    if ($form->{templates}) {
      $dir = "$form->{templates}";
      unlink <$dir/*.html>;
      rmdir "$dir";
    }
  }
  
  # delete config file for user
  $_ = "$userspath/$form->{login}.conf";
  unlink;
    
  $form->redirect;
  
}


sub login_name {
  my $login = shift;
  
  $login =~ s/\[\]//g;
  return ($login) ? $login : undef;
  
}



sub get_value {
  my $line = shift;
  
  my ($null, $value) = split(/=/, $line, 2);

  # remove comments
  $value =~ s/#.*//g;
      
  # remove any trailing whitespace
  $value =~ s/^\s*(.*?)\s*$/$1/;

  $value;
}



sub change_admin_password {

  &check_password;
  
  print qq|Content-type: text/html

<head>
  <title>SQL-Ledger Administration / Change Admin's Password</title>
</head>

<body bgcolor="#FBFFE7">

<h2>Change Admin's Password</h2>

<form method=post action=$form->{script}>

<b>Password</b> <input type=password name=password size=8 value=$form->{rpw}>

<input type=hidden name=root value="$form->{root}">
<input type=hidden name=path value=$form->{path}>
<input type=hidden name=rpw value=$form->{rpw}>

<p>
<input class=submit type=submit name=action value="Change Password">

</form>
|;

}


sub change_password {

  &check_password;
  
  $root->{password} = $form->{password};
  
  if ($form->{rpw} ne $form->{password}) {
    $rpw = $form->{password};
    $form->{rpw} = $form->{password};
  }
  
  $form->{callback} = "$form->{script}?action=list_users&path=$form->{path}&root=$form->{root}&rpw=$form->{rpw}";

  $root->{'root login'} = 1;
  $root->save_member($memberfile);

  $form->{rpw} = $rpw;

  $form->redirect;

}


sub check_password {

  $root = new User "$memberfile", $form->{root};
  
  if ($root->{password}) {
    if ($root->{password} ne crypt $form->{rpw}, substr($form->{root}, 0, 2)) {
      $form->error("Incorrect Password!");
    }
  }
}


sub pg_database_administration {

  $form->{dbdriver} = 'Pg';
  &dbselect_source;

}


sub oracle_database_administration {
  
  $form->{dbdriver} = 'Oracle';

  &dbselect_source;

}


sub dbdriver_defaults {

  # load some defaults for the selected driver
  %driverdefaults = ( 'Pg' => { dbport => '',    # 5432,
                                dbuser => 'postgres',
		             dbdefault => 'template1',
			         label => 'Database',
				dbhost => ''
			      },
                  'Oracle' => { dbport => '',    # 1521,
		                dbuser => 'oralin',
		             dbdefault => 'system',
			         label => 'Tablespace',
				dbhost => 'localhost'
			      }
                    );

  map { $form->{$_} = $driverdefaults{$form->{dbdriver}}{$_} } keys %{ $driverdefaults{Pg} };
  
}
  

sub dbselect_source {

  &check_password;
  
  &dbdriver_defaults;

  $msg{Pg} = "<br>Leave host and port field empty unless you want to make a remote connection.";
  $msg{Oracle} = "<br>You need to enter a hostname even for local connections.";
  

  $form->{title} = "SQL-Ledger Accounting / Database Administration";
  
 
  print qq|Content-type: text/html

<head>
  <title>$form->{title}</title>
</head>

<body bgcolor="#FBFFE7">

<center>
<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<table>
<tr><td>

<table border=1>
  <tr bgcolor=336666>
    <th colspan=4><font color=ffffff>Database</th>
  </tr>

<input type=hidden name=dbdriver value=$form->{dbdriver}>

  <tr>
    <th align=right>Host</th>
    <td><input name=dbhost size=25></td>
    <th align=right>Port</th>
    <td><input name=dbport size=5 value=$form->{dbport}></td>
  </tr>

  <tr>
    <th align=right>Database Administrator</th>
    <td><input name=dbuser size=10 value=$form->{dbuser}></td>
    <th align=right>Password</th>
    <td><input type=password name=dbpasswd size=10></td>
  </tr>
  
  <tr>
    <th align=right>Connect to</th>
    <td colspan=3><input name=dbdefault size=10 value=$form->{dbdefault}></td>
  </tr>

</table>

<input name=callback type=hidden value="$form->{script}?action=list_users&path=$form->{path}&root=$form->{root}&rpw=$form->{rpw}">
<input type=hidden name=root value="$form->{root}">
<input type=hidden name=path value=$form->{path}>
<input type=hidden name=rpw value=$form->{rpw}>

<input type=hidden name=label value=$form->{label}>

<input class=submit type=submit name=action value="Create $form->{label}">
<input class=submit type=submit name=action value="Delete $form->{label}">

</form>

</td></tr>
</table>

<p>This is a preliminary check for existing sources. Nothing will be
created or deleted at this stage!

$msg{$form->{dbdriver}}

|;

}


sub continue {

  &{ $form->{nextsub} };

}


sub create_tablespace {

  &create_database;

}


sub delete_tablespace {

  &delete_database;

}


sub create_database {

  &check_password;

  foreach $item (sort User->dbsources(\%$form)) {
    $dbsources .= "$item<br>";
  }

  opendir SQLDIR, "sql/." || $form-error($!);
  foreach $item (sort grep /-chart\.sql/, readdir SQLDIR) {
    next if ($item eq 'Default-chart.sql');
    $item =~ s/-chart\.sql//;
    $charts .= qq|<input name=chart class=radio type=radio value="$item">&nbsp;$item|;
  }
  closedir SQLDIR;
  $charts = qq|<input name=chart class=radio type=radio value="Default" checked>&nbsp;Default
$charts|;
  
  $form->{title} = qq|SQL-Ledger Database Administration / Create $form->{label}|;
  print qq|Content-type: text/html

<head>
  <title>$form->{title}</title>
</head>

<body bgcolor="#FBFFE7">

<center>
<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<table border=1>
  <tr bgcolor=336666>
    <th colspan=2><font color=ffffff>&nbsp;</th>
  </tr>

  <tr>
    <th align=right>Existing $form->{label}s:</th>
    <td>
    $dbsources
    </td>
  </tr>
  
  <tr>
    <th align=right>Create $form->{label}:</th>
    <td><input name=db></td>
  </tr>

  <tr>
    <th align=right>Create Chart of Accounts:</th>
    <td>$charts</td>
  </tr>
  
  <tr><td colspan=2>
<p>
<input type=hidden name=dbdriver value=$form->{dbdriver}>
<input type=hidden name=dbuser value=$form->{dbuser}>
<input type=hidden name=dbhost value=$form->{dbhost}>
<input type=hidden name=dbport value=$form->{dbport}>
<input type=hidden name=dbpasswd value=$form->{dbpasswd}>
<input type=hidden name=dbdefault value=$form->{dbdefault}>

<input name=callback type=hidden value="$form->{script}?action=list_users&path=$form->{path}&root=$form->{root}&rpw=$form->{rpw}">

<input type=hidden name=root value="$form->{root}">
<input type=hidden name=path value=$form->{path}>
<input type=hidden name=rpw value=$form->{rpw}>

<input type=hidden name=label value=$form->{label}>

<input type=hidden name=nextsub value=dbcreate>

<input class=submit type=submit name=action value="Continue">

  </td></tr>
</table>

</form>

|;

}


sub dbcreate {

  $form->isblank("db", "$form->{label} missing!");

  User->dbcreate(\%$form);
  
  $form->{title} = qq|SQL-Ledger Database Administration / Create $form->{label}|;
  
  print qq|Content-type: text/html

<head>
  <title>$form->{title}</title>
</head>

<body bgcolor="#FBFFE7">

<center>
<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

$form->{label} $form->{db} successfully created!

<input type=hidden name=root value="$form->{root}">
<input type=hidden name=path value="$form->{path}">
<input type=hidden name=rpw value="$form->{rpw}">

<input type=hidden name=nextsub value=list_users>

<p><input class=submit type=submit name=action value=Continue>
</form>
|;

}


sub delete_database {

  &check_password;

  if (@dbsources = User->dbsources_unused(\%$form, $memberfile)) {
    foreach $item (@dbsources) {
      $dbsources .= "<input name=db class=radio type=radio value=$item> $item<br>\n";
    }
  } else {
    $form->error("All $form->{label}s in use by members");
  }

  $form->{title} = qq|SQL-Ledger Database Administration / Delete $form->{label}|;
  
  print qq|Content-type: text/html

<head>
  <title>$form->{title}</title>
</head>

<body bgcolor="#FBFFE7">

<center>
<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<table border=1>
  <tr bgcolor=336666>
    <th><font color=ffffff>The following $form->{label}s are not in use and can be deleted</th>
  </tr>

  <tr>
    <td>
    $dbsources
    </td>
  </tr>
  
  <tr><td>
<p>
<input type=hidden name=dbdriver value=$form->{dbdriver}>
<input type=hidden name=dbuser value=$form->{dbuser}>
<input type=hidden name=dbhost value=$form->{dbhost}>
<input type=hidden name=dbport value=$form->{dbport}>
<input type=hidden name=dbpasswd value=$form->{dbpasswd}>
<input type=hidden name=dbdefault value=$form->{dbdefault}>

<input name=callback type=hidden value="$form->{script}?action=list_users&path=$form->{path}&root=$form->{root}&rpw=$form->{rpw}">

<input type=hidden name=root value="$form->{root}">
<input type=hidden name=path value="$form->{path}">
<input type=hidden name=rpw value="$form->{rpw}">

<input type=hidden name=label value=$form->{label}>

<input type=hidden name=nextsub value=dbdelete>

<input class=submit type=submit name=action value="Continue">

  </td></tr>
</table>

</form>

<p>Select a $form->{label} to delete and press "Continue"
|;

}


sub dbdelete {

  unless ($form->{db}) {
    $form->error("No $form->{label} selected!");
  }

  User->dbdelete(\%$form);

  $form->{title} = qq|SQL-Ledger Database Administration / Delete $form->{label}|;
  
  print qq|Content-type: text/html

<head>
  <title>$form->{title}</title>
</head>

<body bgcolor="#FBFFE7">

<center>
<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

$form->{label} $form->{db} successfully deleted!

<input type=hidden name=root value="$form->{root}">
<input type=hidden name=path value="$form->{path}">
<input type=hidden name=rpw value="$form->{rpw}">

<input type=hidden name=nextsub value=list_users>

<p><input class=submit type=submit name=action value=Continue>
</form>
|;

}


