#=====================================================================
# SQL-Ledger Accounting
# Copyright (c) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
#
# Accounts Payables
#
# CHANGE LOG:
#   DS. 2000-07-04  Created
#   DS. 2001-07-24  added number formatting
#   DS. 2001-08-19  due date and tax calculation
#
#======================================================================


use DBI;
use SL::Form;
use SL::AP;
use SL::IR;


$form = new Form;


if ($form->{action}) {
  require("$userspath/$form->{login}.conf");
  $myconfig{dbpasswd} = unpack 'u', $myconfig{dbpasswd};
  map { $form->{$_} = $myconfig{$_} } qw(stylesheet charset);

  $locale = new Locale $myconfig{countrycode}, "ap";
  &{ $locale->findsub($form->{action}) };
} else {
  $form->error("action= not defined!");
}

# end of main


# this is for our long dates
# $locale->text('January')
# $locale->text('February')
# $locale->text('March')
# $locale->text('April')
# $locale->text('May')
# $locale->text('June')
# $locale->text('July')
# $locale->text('August')
# $locale->text('September')
# $locale->text('October')
# $locale->text('November')
# $locale->text('December')

# this is for our short month
# $locale->text('Jan')
# $locale->text('Feb')
# $locale->text('Mar')
# $locale->text('Apr')
# $locale->text('May')
# $locale->text('Jun')
# $locale->text('Jul')
# $locale->text('Aug')
# $locale->text('Sep')
# $locale->text('Oct')
# $locale->text('Nov')
# $locale->text('Dec')


sub add {

  $form->{title} = "Add";
  $form->{callback} = "$form->{script}?path=$form->{path}&action=add&login=$form->{login}";

  &create_links;
  &display_form;
  
}


sub edit {
  
  $form->{title} = "Edit";

  &create_links;
  &display_form;

}


sub display_form {
  
  &form_header;
  &form_footer;

}


sub create_links {
  
  $form->create_links("AP", \%myconfig, "vendor");

  $taxincluded = $form->{taxincluded};
  
  # build the popup menus
  foreach $ref (@{ $form->{all_vendor} }) {
    if ($ref->{id} == $form->{vendor_id}) {
      $form->{vendor} = "$ref->{name}--$ref->{id}";
      $form->{vendor_id} = $ref->{id};
      IR->get_vendor(\%myconfig, \%$form);
    }
    $form->{selectvendor} .= "<option>$ref->{name}--$ref->{id}\n";
  }

  foreach $key (keys %{ $form->{AP_links} }) {
    foreach $ref (@{ $form->{AP_links}{$key} }) {

      $nkey = $key;
      
      if ($key eq "AP_tax") {
	$nkey = "${key}_$ref->{accno}";
      }
      
      if ($form->{amount}{$ref->{accno}}) {
	# if there is a value we have an old entry
	$form->{$nkey} = "$ref->{accno}--$ref->{description}";
	$form->{amount}{$nkey} = $form->{amount}{$ref->{accno}};
      }
      $form->{"select$nkey"} .= "<option>$ref->{accno}--$ref->{description}\n";
    }
  }

  
  $form->{paid} = $form->{amount}{AP_paid};

  foreach $item (split / /, $form->{taxaccounts}) {
    $form->{"tax_$item"} = $form->{amount}{"AP_tax_$item"};
    $totaltax += $form->{"tax_$item"};
  }

  $form->{taxincluded} = $taxincluded if ($form->{id});
  $form->{amount}{AP_amount} += $totaltax if ($form->{taxincluded});
  
  # reverse amount
  $form->{amount} = $form->{amount}{AP_amount} * -1;

}


sub form_header {

  $form->{title} = $locale->text($form->{title} .' Accounts Payables');

  $form->{taxincluded} = ($form->{taxincluded}) ? "checked" : "";
  
# this is for our parser only!
# class=submit type=submit $locale->text('Add Accounts Payables')
# class=submit type=submit $locale->text('Edit Accounts Payables')


  # set option selected
  map { $form->{$_} =~ s/ selected// } qw(selectAP selectAP_amount selectAP_paid selectvendor);
  map { $form->{"select${_}"} =~ s/option>$form->{$_}/option selected>$form->{$_}/ } qw(AP AP_amount AP_paid vendor);

  # format amounts
  map { $form->{$_} = $form->format_amount(\%myconfig, $form->{$_}) } qw(amount paid);


  $form->header;

  print qq|

<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<input type=hidden name=id value=$form->{id}>
<input type=hidden name=sort value=$form->{sort}>

<table border=1>
<tr valign=top>
<td>

<table>

<tr>
  <th align=right>|.$locale->text('Record in').qq|</th>
  <td colspan=3><select name=AP>$form->{selectAP}</select></td>
  <input type=hidden name=selectAP value="$form->{selectAP}">
</tr>

<tr>
  <th align=right>|.$locale->text('Vendor').qq|</th>
  <td colspan=3><select name=vendor>$form->{selectvendor}</select></td>
  <input type=hidden name=selectvendor value="$form->{selectvendor}">
  <input type=hidden name=oldvendor value="$form->{vendor_id}">
  <input type=hidden name=terms value=$form->{terms}>
</tr>

<tr>
  <th align=right>|.$locale->text('Tax Included').qq|</th>
  <td colspan=3><input name=taxincluded class=checkbox type=checkbox value=1 $form->{taxincluded}></td>
</tr>

<tr>
  <th align=right>|.$locale->text('Invoice Number').qq|</th>
  <td colspan=3><input name=invnumber size=20 value="$form->{invnumber}"></td>
</tr>

<tr>
  <th align=right>|.$locale->text('Order Number').qq|</th>
  <td colspan=3><input name=ordnumber size=20 value="$form->{ordnumber}"></td>
</tr>

<tr>
  <th align=right>|.$locale->text('Invoice Date').qq|<br>($myconfig{'dateformat'})</th>
  <td><input name=transdate size=10 value=$form->{transdate}></td>
  <th align=right>|.$locale->text('Due Date').qq|<br>($myconfig{'dateformat'})</th>
  <td><input name=duedate size=10 value=$form->{duedate}></td>
</tr>

<tr>
  <th align=right>|.$locale->text('Amount').qq|</th>
  <td><input name=amount size=10 value=$form->{amount}></td>
  <td colspan=2><select name=AP_amount>$form->{selectAP_amount}</select></td>
  <input type=hidden name=selectAP_amount value="$form->{selectAP_amount}">
</tr>

<input type=hidden name=taxaccounts value="$form->{taxaccounts}">
|;


  $taxlabel = ($form->{taxincluded}) ? $locale->text('Tax Included') : $locale->text('Tax');
  
  foreach $item (split / /, $form->{taxaccounts}) {

    # format and reverse tax
    $form->{"tax_$item"} = $form->format_amount(\%myconfig, $form->{"tax_$item"} * -1);
 
    print qq|
<tr>
<th align=right>${taxlabel}</th>
<td><input name="tax_$item" size=9 value=$form->{"tax_$item"}></td>
<td colspan=2><select name=AP_tax_$item>$form->{"selectAP_tax_$item"}</select></td>
</tr>

<input type=hidden name="${item}_rate" value="$form->{"${item}_rate"}">
<input type=hidden name="${item}_description" value="$form->{"${item}_description"}">
<input type=hidden name="selectAP_tax_$item" value="$form->{"selectAP_tax_$item"}">
|;

  }
   
  
  print qq|
<tr>
  <th align=right>|.$locale->text('Paid').qq|</th>
  <td><input name=paid size=10 value=$form->{paid}></td>
  <td colspan=2><select name=AP_paid>$form->{selectAP_paid}</select></td>
  <input type=hidden name=selectAP_paid value="$form->{selectAP_paid}">
</tr>

<tr>
  <th align=right>|.$locale->text('Date Paid').qq| ($myconfig{dateformat})</th>
  <td><input name=datepaid size=10 value=$form->{datepaid}></td>
</tr>
</table>

</td>
</tr>

|;

}


sub form_footer {

  print qq|

<input name=callback type=hidden value="$form->{callback}">

<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

</table>

<br>
<input class=submit type=submit name=action value="|.$locale->text('Update Form').qq|">
<input class=submit type=submit name=action value="|.$locale->text('Post Transaction').qq|">
|;

  if ($form->{id}) {
    print qq|<input class=submit type=submit name=action value="|.$locale->text('Delete Transaction').qq|">|;
  }

  print "</form>";

}


sub update_form {

  # split vendor
  ($null, $form->{vendor_id}) = split /--/, $form->{vendor};

  # recalculate form
  if ($form->{oldvendor} != $form->{vendor_id}) {
    IR->get_vendor(\%myconfig, \%$form);
  }

  # recalculate due date
  $form->{duedate} = $form->current_date(\%myconfig, $form->{transdate}, $form->{terms});
  
  $form->{amount} = $form->parse_amount(\%myconfig, $form->{amount});

  # recalculate taxes
  if ($form->{taxincluded}) {
    $taxrate = 0;
    map { $taxrate += $form->{"${_}_rate"} } split / /, $form->{taxaccounts};
    # proportion taxes
    
    foreach $item (split / /, $form->{taxaccounts}) {
      $form->{"tax_$item"} = sprintf("%.2f", $form->{amount} * (1 - 1 / (1 + $taxrate)) * $form->{"${item}_rate"} / $taxrate) * -1;
      $form->{"selectAP_tax_$item"} = qq|<option>$item--$form->{"${item}_description"}|;
      $totaltax -= $form->{"tax_$item"};
    }
  } else {
    foreach $item (split / /, $form->{taxaccounts}) {
      $form->{"tax_$item"} = $form->{amount} * $form->{"${item}_rate"} * -1;
      $form->{"selectAP_tax_$item"} = qq|<option>$item--$form->{"${item}_description"}|;
      $totaltax -= $form->{"tax_$item"};
    }
  }
 
  $form->{paid} = ($form->{taxincluded}) ? $form->{amount} : $form->{amount} + $totaltax;
  $form->{datepaid} = $form->{transdate};

  &display_form;

}
 
 
sub post_transaction {

  # check if there is an invoice number, invoice and due date
  $form->isblank("invnumber", $locale->text("Invoice Number missing!"));
  $form->isblank("transdate", $locale->text("Invoice Date missing!"));
  $form->isblank("duedate", $locale->text("Due Date missing!"));
  
  # split vendor
  ($null, $form->{vendor_id}) = split /--/, $form->{vendor};
  
  # if the vendor changed get new vendor
  if ($form->{oldvendor} != $form->{vendor_id}) {
    IR->get_vendor(\%myconfig, \%$form);
  }

  $form->isposted(AP->post_transaction(\%myconfig, \%$form));
  
}



sub delete_transaction {

  $form->{title} = $locale->text('Confirm!');
  
  $form->header;

  print qq|<form method=post action=$form->{script}>\n|;

  foreach $key (keys %$form) {
    print qq|<input type=hidden name=$key value="$form->{$key}">\n|;
  }

  print qq|
<h2><font color=red>$form->{title}</font></h2>


<h4>|.$locale->text('Are you sure you want to delete Transaction').qq| $form->{invnumber}</h4>
<p>

<input name=action class=submit type=submit value="|.$locale->text('Yes Delete Transaction').qq|">
</form>
|;

}



sub yes_delete_transaction {

  $form->isdeleted(AP->delete_transaction(\%myconfig, \%$form));

}


sub search {
  
  # setup vendor selection
  $form->all_vc(\%myconfig, "vendor");
  
  my $selection = "<option selected>" . $locale->text('All Vendors') ."\n";
  
  foreach $ref (@{ $form->{all_vendor} }) {
    $selection .= "<option>$ref->{name}--$ref->{id}\n";
  }
  
  if ($form->{nextsub} eq "payments") {
    $form->{title} = $locale->text('Process Payments');
    $listopen = qq|
    <input type=hidden name=sort value=invnumber>
    <input type=hidden name=open value=Y>|;
  }
  if ($form->{nextsub} eq "ap_transactions") {
    $form->{title} = $locale->text('Search for Transaction');
    $listopen = qq|
<input type=hidden name=sort value=transdate>
<tr>
<th align=right>|.$locale->text('List open transaction').qq|</th>
<td><input name=open class=checkbox type=checkbox value=Y></td>
</tr>
<tr>
<th align=right>|.$locale->text('Include in Report').qq|</th>
<td colspan=3>
<input name="l_invnumber" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Invoice Number').qq|
<input name="l_ordnumber" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Order Number').qq|
<input name="l_transdate" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Invoice Date').qq|
<input name="l_duedate" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Due Date').qq|
<input name="l_name" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Vendor').qq|
<input name="l_netamount" class=checkbox type=checkbox value=Y>&nbsp;|.$locale->text('Net Amount').qq|
<input name="l_tax" class=checkbox type=checkbox value=Y>&nbsp;|.$locale->text('Tax').qq|
<input name="l_amount" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Total').qq|
<input name="l_paid" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Paid').qq|
<input name="l_datepaid" class=checkbox type=checkbox value=Y>&nbsp;|.$locale->text('Date Paid').qq|
<br><input name="l_subtotal" class=checkbox type=checkbox value=Y>&nbsp;|.$locale->text('Subtotal').qq|
</td></tr>
|;
  }
    
  $form->header;

  print  qq|<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<table border=1>
<tr><td>

<table>

<tr>
<th align=right>|.$locale->text('Vendor').qq|</th><td colspan=3><select name=vendor>$selection</select></td>
</tr>

<tr>
<th align=right>|.$locale->text('Invoice Number').qq|</th><td><input name=invnumber size=20></td>
</tr>

<tr>
<th align=right>|.$locale->text('Order Number').qq|</th><td><input name=ordnumber size=20></td>
</tr>

<tr>
<th align=right>|.$locale->text('From').qq| ($myconfig{dateformat})</th><td><input name=transdatefrom size=10></td>
<th align=right>|.$locale->text('To').qq|</th><td><input name=transdateto size=10></td>
</tr>

$listopen

</table>

</td></tr>
</table>

<p>
<input type=hidden name=nextsub value=$form->{nextsub}>
<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

<input class=submit type=submit name=action value="|.$locale->text('Search for Transaction').qq|">
</form>

<p>|.$locale->text('To display all transactions, leave fields blank');

}


sub search_for_transaction {

  &{ $form->{nextsub} };

}


sub ap_transactions {
    

  # split vendor
  ($vendor, $form->{vendor_id}) = split(/--/, $form->{vendor});
  
  # $form->{vendor_id} is needed in AP->ap_transactions
  AP->ap_transactions(\%myconfig, \%$form);

  # construct href
  $href = "$form->{script}?path=$form->{path}&action=ap_transactions&login=$form->{login}&transdatefrom=$from->{transdatefrom}&transdateto=$form->{transdateto}&open=$form->{open}&invnumber=" . $form->escape($form->{invnumber}) . "&ordnumber=" . $form->escape($form->{ordnumber}) . "&vendor=" . $form->escape($form->{vendor});

  # construct callback
  $vendor = $form->{vendor};
  $vendor =~ s/&/_/g;
  $callback = "$form->{script}?path=$form->{path}&action=ap_transactions&login=$form->{login}&transdatefrom=$from->{transdatefrom}&transdateto=$form->{transdateto}&open=$form->{open}&invnumber=$form->{invnumber}&ordnumber=$form->{ordnumber}&vendor=$vendor";

  @columns = $form->sort_columns(qw(transdate duedate invnumber ordnumber name netamount tax amount paid datepaid));

  foreach $item (@columns) {
    if ($form->{"l_$item"} eq "Y") {
      push @column_index, $item;
      
      # add column to href and callback
      $callback .= "&l_$item=Y";
      $href .= "&l_$item=Y";
    }
  }
  
  if ($form->{l_subtotal} eq 'Y') {
    $callback .= "&l_subtotal=Y";
    $href .= "&l_subtotal=Y";
  }
  
    
  $column_header{transdate} = qq|<th><a href=$href&sort=transdate><font color=ffffff>|.$locale->text('Date').qq|</a></th>|;
  $column_header{duedate} = qq|<th><a href=$href&sort=duedate><font color=ffffff>|.$locale->text('Due').qq|</a></th>|;
  $column_header{invnumber} = qq|<th><a href=$href&sort=invnumber><font color=ffffff>|.$locale->text('Invoice').qq|</a></th>|;
  $column_header{ordnumber} = qq|<th><a href=$href&sort=ordnumber><font color=ffffff>|.$locale->text('Order').qq|</a></th>|;
  $column_header{name} = qq|<th><a href=$href&sort=name><font color=ffffff>|.$locale->text('Vendor').qq|</a></th>|;
  $column_header{netamount} = qq|<th><font color=ffffff>|.$locale->text('Net Amount').qq|</th>|;
  $column_header{tax} = qq|<th><font color=ffffff>|.$locale->text('Tax').qq|</th>|;
  $column_header{amount} = qq|<th><font color=ffffff>|.$locale->text('Amount').qq|</th>|;
  $column_header{paid} = qq|<th><font color=ffffff>|.$locale->text('Paid').qq|</th>|;
  $column_header{datepaid} = qq|<th><a href=$href&sort=datepaid><font color=ffffff>|.$locale->text('Date Paid').qq|</a></th>|;

  $form->{title} = $locale->text('AP Transactions');
  
  ($vendor) = split /--/, $form->{vendor};
  if ($form->{vendor_id}) {
    $option .= $vendor;
  }
  if ($form->{transdatefrom}) {
    $option .= "<br>".$locale->text('From').": ".$locale->date(\%myconfig, $form->{transdatefrom}, 1);
  }
  if ($form->{transdateto}) {
    $option .= "<br>".$locale->text('To').": ".$locale->date(\%myconfig, $form->{transdateto}, 1);
  }

 
  $form->header;
  
  print qq|
<h2>$form->{title}</h2>

<b>$option</b>

<p>
<table border=1>
<tr bgcolor=336666>
|;

map { print "$column_header{$_}\n" } @column_index;

print qq|
</tr>
|;

  # add sort and escape callback
  $callback = $form->escape($callback . "&sort=$form->{sort}");

  $sameitem = $form->{AP}->[0]->{$form->{sort}};

  # sums and tax on reports by Antonio Gallardo
  #
  foreach $ap (@{ $form->{AP} }) {

    if ($form->{l_subtotal} eq 'Y') {
      if ($sameitem ne $ap->{$form->{sort}}) {
	&ap_subtotal;
      }
    }
    
    $column_data{netamount} = "<td align=right>&nbsp;" . $form->format_amount(\%myconfig, $ap->{netamount}) . "</td>";
    $column_data{tax} = "<td align=right>&nbsp;" . $form->format_amount(\%myconfig, $ap->{amount} - $ap->{netamount}) . "</td>";
    $column_data{amount} = "<td align=right>&nbsp;" . $form->format_amount(\%myconfig, $ap->{amount}) . "</td>";
    $column_data{paid} = "<td align=right>&nbsp;" . $form->format_amount(\%myconfig, $ap->{paid}) . "</td>";

    $totalnetamount += $ap->{netamount};
    $totalamount += $ap->{amount};
    $totalpaid += $ap->{paid};

    $subtotalnetamount += $ap->{netamount};
    $subtotalamount += $ap->{amount};
    $subtotalpaid += $ap->{paid};

    $column_data{transdate} = "<td>$ap->{transdate}&nbsp;</td>";
    $column_data{duedate} = "<td>$ap->{duedate}&nbsp;</td>";
    $column_data{datepaid} = "<td>$ap->{datepaid}&nbsp;</td>";

    $module = ($ap->{invoice}) ? "ir.pl" : $form->{script};
    $column_data{invnumber} = "<td><a href=$module?path=$form->{path}&action=edit&id=$ap->{id}&login=$form->{login}&callback=$callback>$ap->{invnumber}</a></td>";
    $column_data{ordnumber} = "<td>&nbsp;$ap->{ordnumber}</td>";
    $column_data{name} = "<td>$ap->{name}</td>";

    print "<tr>";
    
    map { print "$column_data{$_}\n" } @column_index;

    print qq|</tr>|;

  }
  
  if ($form->{l_subtotal} eq 'Y') {
    &ap_subtotal;
  }
  
  # print totals
  print qq|<tr bgcolor=000000>|;
  
  map { $column_data{$_} = "<td>&nbsp;</td>" } @column_index;
  
  $column_data{netamount} = "<th align=right><font color=ffffff>&nbsp;" . $form->format_amount(\%myconfig, $totalnetamount) . "</th>";
  $column_data{tax} = "<th align=right><font color=ffffff>&nbsp;" . $form->format_amount(\%myconfig, $totalamount - $totalnetamount) . "</th>";
  $column_data{amount} = "<th align=right><font color=ffffff>&nbsp;" . $form->format_amount(\%myconfig, $totalamount) . "</th>";
  $column_data{paid} = "<th align=right><font color=ffffff>&nbsp;" . $form->format_amount(\%myconfig, $totalpaid) . "</th>";

  map { print "$column_data{$_}\n" } @column_index;

  print qq|</tr>
  
</table>

<p>|.$locale->text('Click on table headers to sort table').qq|
<br>|.$locale->text('Click on invoice number to edit transaction');

}


sub ap_subtotal {

  map { $column_data{$_} = "<td>&nbsp;</td>" } @column_index;
  
  $column_data{netamount} = "<th align=right><font color=ffffff>&nbsp;" . $form->format_amount(\%myconfig, $subtotalnetamount) . "</th>";
  $column_data{tax} = "<td align=right>&nbsp;" . $form->format_amount(\%myconfig, $subtotalamount - $subtotalnetamount) . "</th>";
  $column_data{amount} = "<th align=right><font color=ffffff>&nbsp;" . $form->format_amount(\%myconfig, $subtotalamount) . "</th>";
  $column_data{paid} = "<th align=right><font color=ffffff>&nbsp;" . $form->format_amount(\%myconfig, $subtotalpaid) . "</th>";

  $subtotalnetamount = 0;
  $subtotalamount = 0;
  $subtotalpaid = 0;

  print "<tr bgcolor=333366>
";
  
  map { print "$column_data{$_}\n" } @column_index;

  print qq|</tr>
|;

}


sub payments {

  # split vendor
  ($vendor, $form->{vendor_id}) = split(/--/, $form->{vendor});
  
  # $form->{vendor_id} is needed in AP->payments
  AP->payments(\%myconfig, \%$form);

  # construct href
  $href = "$form->{script}?path=$form->{path}&action=payments&login=$form->{login}&transdatefrom=$from->{transdatefrom}&transdateto=$form->{transdateto}&open=$form->{open}&invnumber=" . $form->escape($form->{invnumber}) . "&ordnumber=" . $form->escape($form->{ordnumber}) . "&vendor=" . $form->escape($form->{vendor});


  @column_index = $form->sort_columns(qw(invnumber transdate ordnumber name));

  $column_header{transdate} = qq|<th><a href=$href&sort=transdate><font color=ffffff>|.$locale->text('Date').qq|</a></th>|;
  $column_header{invnumber} = qq|<th><a href=$href&sort=invnumber><font color=ffffff>|.$locale->text('Invoice').qq|</a></th>|;
  $column_header{ordnumber} = qq|<th><a href=$href&sort=ordnumber><font color=ffffff>|.$locale->text('Order').qq|</a></th>|;
  $column_header{name} = qq|<th><a href=$href&sort=name><font color=ffffff>|.$locale->text('Vendor').qq|</a></th>|;

  $form->{title} = $locale->text('Process Payments');
  
  $form->header;
    
  print qq|
<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<table border=0>

<tr bgcolor=336666>
|;

map { print "$column_header{$_}\n" } @column_index;

  print qq|
<th><font color=ffffff>|.$locale->text('Amount').qq|</th>
</tr>
|;

  # construct and escape callback
  $vendor = $form->{vendor};
  $vendor =~ s/&/_/g;
  $callback = $form->escape("$form->{script}?path=$form->{path}&action=payments&transdatefrom=$from->{transdatefrom}&transdateto=$form->{transdateto}&open=$form->{open}&login=$form->{login}&sort=$form->{sort}&vendor=$vendor&invnumber=$form->{invnumber}&ordnumber=$form->{ordnumber}");
  

  # build the popup menu for the accounts
  foreach $accno (sort keys %{$form->{paidaccount}}) {
    $paidaccount .= qq|<option>$form->{paidaccount}{$accno}\n|;
  }

  foreach $ap (@{ $form->{AP} }) {
    $i++;
    $amount = $form->format_amount(\%myconfig, $ap->{amount});
    $paid = $form->format_amount(\%myconfig, $ap->{paid});

    print "<tr>";
    
    $column_data{transdate} = qq|<td><input type=hidden name="transdate_$i" value=$ap->{transdate}>$ap->{transdate}</td>|;

if ($ap->{invoice}) {
  $column_data{invnumber} = "<td><a href=ir.pl?path=$form->{path}&action=edit&id=$ap->{id}&login=$form->{login}&callback=$callback>$ap->{invnumber}</a></td>";
} else {
  $column_data{invnumber} = "<td><a href=$form->{script}?path=$form->{path}&action=edit&id=$ap->{id}&login=$form->{login}&callback=$callback>$ap->{invnumber}</a></td>";
}

    $column_data{"ordnumber"} = "<td>&nbsp;$ap->{ordnumber}</td>";
    $column_data{name} = "<td>$ap->{name}</td>";

    map { print "$column_data{$_}\n" } @column_index;

    print qq|
<td align=right>$amount</td>
</tr>
<tr>
|;
  
      
    # arrange data for second row
    $column_data{invnumber} = qq|<th>|.$locale->text('PAYMENT').qq|</th>|;
    $column_data{transdate} = qq|<td><font size=-1><input name="datepaid_$i" size=10 value=$ap->{datepaid}></font></td>|;
    $column_data{ordnumber} = "<td>&nbsp;</td>";
    
    
    $column_data{name} = qq|<td><select name="accno_$i">\n|;

    # add the other accounts
    $column_data{name} .= $paidaccount;
    
    
    if ($ap->{paidaccno}) {
      $column_data{name} =~ s/<option>$form->{paidaccount}{$ap->{paidaccno}}/<option selected>$form->{paidaccount}{$ap->{paidaccno}}/;
      print qq|<input type=hidden name="old_accno_$i" value=$ap->{paidaccno}>\n|;
    }

    $column_data{name} .= qq|</select></td>|;
    
    map { print "$column_data{$_}\n" } @column_index;

    print qq|
<td><font size=-1><input name="paid_$i" size=9 value=$paid></font></td>
</tr>

<input type=hidden name="id_$i" value=$ap->{id}>
<input type=hidden name="amount_$i" value=$ap->{amount}>
<input type=hidden name="AP_$i" value=$ap->{AP_account}>
<input type=hidden name="old_paid_$i" value=$ap->{paid}>
<input type=hidden name="old_accno_$i" value=$ap->{paidaccno}>

<tr><td colspan=5><hr size=3 noshade></td></tr>
|;

  }
  
  print qq|
</table>

<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

<input type=hidden name=rowcount value=$i>

<p>
<input class=submit type=submit name=action value="|.$locale->text('Process Payments').qq|">

</form>

<p>|.$locale->text('Click on table headers to sort table').qq|
<br>|.$locale->text('Click on invoice number to edit transaction');

}


sub process_payments {

  # construct callback
  $form->{callback} = "$form->{script}?action=search&nextsub=payments&login=$form->{login}&path=$form->{path}";

  AP->process_payments(\%myconfig, \%$form);
  $form->redirect;
 
}


