#=====================================================================
# SQL-Ledger Accounting
# Copyright (c) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
#
# Inventory invoicing module
#
# CHANGE LOG:
#   DS. 2000-07-04  Created
#   DS. 2001-06-14  Changed accno links to chart_id
#   DS. 2001-07-01  Changed Add Item and included in Update Form
#                   Added linetotal
#   DS. 2001-07-25  number formatting
#   DS. 2001-07-28  assembly invoicing
#
#======================================================================


use DBI;
use SL::Form;
use SL::IS;

$form = new Form;


if ($form->{action}) {
  require("$userspath/$form->{login}.conf");
  $myconfig{dbpasswd} = unpack 'u', $myconfig{dbpasswd};
  map { $form->{$_} = $myconfig{$_} } qw(stylesheet charset);

  $locale = new Locale $myconfig{countrycode}, "is";
  
  &{ $locale->findsub($form->{action}) };
} else {
  $form->error("action= not defined!");
}

# end of main


# this is for our long dates
# $locale->text('January')
# $locale->text('February')
# $locale->text('March')
# $locale->text('April')
# $locale->text('May')
# $locale->text('June')
# $locale->text('July')
# $locale->text('August')
# $locale->text('September')
# $locale->text('October')
# $locale->text('November')
# $locale->text('December')

# this is for our short month
# $locale->text('Jan')
# $locale->text('Feb')
# $locale->text('Mar')
# $locale->text('Apr')
# $locale->text('May')
# $locale->text('Jun')
# $locale->text('Jul')
# $locale->text('Aug')
# $locale->text('Sep')
# $locale->text('Oct')
# $locale->text('Nov')
# $locale->text('Dec')


sub add {

  $form->{title} = $locale->text('Add Customer Invoice');

  $form->{callback} = "$form->{script}?action=add&path=$form->{path}&login=$form->{login}";

  &create_links;
  
  &prepare_invoice;
  
}


sub continue {

  &{ $form->{nextsub} };

}


sub create_links {

  # create links
  $form->create_links("AR", \%myconfig, "customer");
  
  # build the popup menus
  foreach $ref (@{ $form->{all_customer} }) {
    if ($ref->{id} == $form->{customer_id}) {
      $form->{customer} = "$ref->{name}--$ref->{id}";
      IS->get_customer(\%myconfig, \%$form);
    }
    $form->{selectcustomer} .= "<option>$ref->{name}--$ref->{id}\n";
  }


  foreach $key (keys %{ $form->{AR_links} }) {
    foreach $ref (@{ $form->{AR_links}{$key} }) {
      if ($form->{amount}{$ref->{accno}}) {
	$form->{$key} = "$ref->{accno}--$ref->{description}";
        if ($key eq 'AR_paid') {
	  $form->{paid} = $form->{amount}{$ref->{accno}} * -1;
	}
      }
      $form->{"select$key"} .= "<option>$ref->{accno}--$ref->{description}\n";
    }
  }


}



sub prepare_invoice {

  # gets invoice or default dates
  IS->retrieve_invoice(\%myconfig, \%$form);


  if ($form->{id}) {
    
    map { $form->{$_} =~ s/"/&quot;/g } qw(invnumber shippingpoint notes);
    
    $form->{creditlimit} = $form->format_amount(\%myconfig, $form->{creditlimit});

    foreach $ref (@{ $form->{invoice_details} } ) {
      $i++;
      map { $form->{"${_}_$i"} = $ref->{$_} } keys %{ $ref };
      $form->{"discount_$i"} = $form->format_amount(\%myconfig, $form->{"discount_$i"} * 100, "", "nat");
      $form->{"sellprice_$i"} = $form->format_amount(\%myconfig, $form->{"sellprice_$i"});
      $form->{"qty_$i"} = $form->format_amount(\%myconfig, $form->{"qty_$i"}, "", "nat");
      
      map { $form->{"${_}_$i"} =~ s/"/&quot;/g } qw(partnumber description unit);
      $form->{rowcount} = $i;
    }
  }

  $form->{new_invoice} = !$form->{id};

  &display_form;
  
}



sub form_header {


  $form->header;

  $form->{taxincluded} = ($form->{taxincluded}) ? "checked" : "";

  # set option selected
  map { $form->{$_} =~ s/ selected// } qw(selectAR selectAR_paid selectcustomer);
  map { $form->{"select${_}"} =~ s/option>$form->{$_}/option selected>$form->{$_}/ } qw(AR AR_paid customer);
  

  print qq|

<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<input type=hidden name=id value=$form->{id}>
<input type=hidden name=new_invoice value=$form->{new_invoice}>
<input type=hidden name=title value="$form->{title}">

<input type=hidden name=customer_id value=$form->{customer_id}>

<input type=hidden name=discount value=$form->{discount}>
<input type=hidden name=creditlimit value=$form->{creditlimit}>
<input type=hidden name=nativecurr value="$form->{nativecurr}">


<table border=1>
<tr valign=top>
<td>
<table callspacing=0 cellpadding=0>

<tr>
  <td colspan=6><b>|.$locale->text('Customer').qq|</b> <select name=customer>$form->{selectcustomer}</select> &nbsp;&nbsp; 
  <b>|.$locale->text('Credit Limit').qq|</b> $form->{creditlimit}
  </td>
</tr>

<input type=hidden name=selectcustomer value="$form->{selectcustomer}">

<tr>
  <td colspan=6>
  <b>|.$locale->text('E-mail').qq|</b>
  <font size=-1> <input name=email size=25 value=$form->{email}></font>
  <b>|.$locale->text('Freight').qq|</b> 
  <font size=-1> <input name=shippingpoint value="$form->{shippingpoint}"></font>
  </td>
</tr>

<tr>
  <td colspan=6><b>|.$locale->text('Record in').qq|</b>
  <select name=AR>$form->{selectAR}</select>
  </td>
  <input type=hidden name=selectAR value="$form->{selectAR}">
</tr>
 
<tr>
  <td colspan=6 nowrap>
  <b>|.$locale->text('Invoice Number').qq|</b> 
  <font size=-1> <input name=invnumber size=10 value="$form->{invnumber}"></font>
  <b>|.$locale->text('Tax Included').qq|</b>
  <input name=taxincluded class=checkbox type=checkbox value=1 $form->{taxincluded}> 
  </td>
</tr>

<tr>
  <td colspan=6 nowrap>
  <b>|.$locale->text('Invoice Date').qq| ($myconfig{dateformat})</b> 
  <font size=-1> <input name=invdate size=10 value=$form->{invdate}></font>
  
  <b>|.$locale->text('Date Due').qq| ($myconfig{dateformat})</b> 
  <font size=-1> <input name=duedate size=10 value=$form->{duedate}></font>
  </td>
</tr>

<input type=hidden name=taxaccounts value="$form->{taxaccounts}">
|;

  foreach $item (split / /, $form->{taxaccounts}) {
    print qq|
<input type=hidden name="${item}_rate" value="$form->{"${item}_rate"}">
<input type=hidden name="${item}_description" value="$form->{"${item}_description"}">
<input type=hidden name="${item}_taxnumber" value="$form->{"${item}_taxnumber"}">
|;
  }

}



sub form_footer {

  $form->{paid} = $form->format_amount(\%myconfig, $form->{paid});
    
  unless ($form->{taxincluded}) {

      $form->{invsubtotal} = $form->format_amount(\%myconfig, $form->{invsubtotal});
      
      $subtotal = qq|
	<tr>
	  <th align=right>|.$locale->text('Subtotal').qq|:</th>
	  <td align=right>$form->{invsubtotal}</td>
	</tr>
	|;
	
    foreach $item (split / /, $form->{taxaccounts}) {
      if ($form->{"${item}_total"}) {
	$form->{"${item}_total"} = $form->format_amount(\%myconfig, $form->{"${item}_total"});
	
	$tax .= qq|
	<tr>
	  <th align=right>$form->{"${item}_description"}:</th>
	  <td align=right>$form->{"${item}_total"}</td>
	</tr>
	|;
      }
    }
  }

 
  $form->{invtotal} = $form->format_amount(\%myconfig, $form->{invtotal});
  
  print qq|

<input name=callback type=hidden value="$form->{callback}">

<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

</table>
</td>
</tr>

<tr>
  <td><table width=100%>
  <tr>
    <td>
      <table>
        $subtotal
	
	$tax
      
	<tr>
	  <th align=right>|.$locale->text('Total').qq|:</th>
	  <td align=right>$form->{invtotal}</td>
	</tr>
      </table>
    </td>
    
    <td>
      <table>
	<tr>
	  <th>|.$locale->text('Notes').qq|</th>
	</tr>

	<tr>
	  <td><font size=-1>
	  <textarea name=notes rows=3 cols=40>$form->{notes}</textarea>
	  </td>
	</tr>
      </table>
    </td>
  </tr>

  <tr><td colspan=3>

</tr>

<tr>
  <td colspan=3><b>|.$locale->text('Paid').qq|</b>
  <input name=paid size=10 value=$form->{paid}></font>
  <select name=AR_paid>$form->{selectAR_paid}</select>
  </td>
  <input type=hidden name=selectAR_paid value="$form->{selectAR_paid}">
</tr>

 
</td></tr></table>

</table>
<br>
<input class=submit type=submit name=action value="|.$locale->text('Update Form').qq|">
<input class=submit type=submit name=action value="|.$locale->text('Print Invoice').qq|">
<input class=submit type=submit name=action value="|.$locale->text('Packing List').qq|">
<input class=submit type=submit name=action value="|.$locale->text('E-mail Invoice').qq|">
<input class=submit type=submit name=action value="|.$locale->text('Post Invoice').qq|">
|;

  if ($form->{id}) {
    print qq|<input class=submit type=submit name=action value="|.$locale->text('Delete Invoice').qq|">|;
  }

  print "</form>";

}



sub display_form {

  &form_header;

  $form->{rowcount}++;
  
  # create rows
  map { &display_row($_) } (1 .. $form->{rowcount});

  &form_footer;
}



sub display_row {
  my $i = shift;

  if ($i == 1) {
    $form->{invsubtotal} = 0;
    $form->{invtotal} = 0;

    map { $form->{"${_}_total"} = 0 } split / /, $form->{taxaccounts};
    
    print qq|
<tr bgcolor=336666>
  <th><font color=ffffff>|.$locale->text('Number').qq|</th>
  <th><font color=ffffff>|.$locale->text('Description').qq|</th>
  <th><font color=ffffff>|.$locale->text('Qty').qq|</th>
  <th><font color=ffffff>|.$locale->text('Unit').qq|</th>
  <th><font color=ffffff>|.$locale->text('Price').qq|</th>
  <th><font color=ffffff>|.$locale->text('Disc').qq|</th>
  <th><font color=ffffff>|.$locale->text('Amount').qq|</th>
</tr>
|;
  }

  # undo formatting
  $form->{"qty_$i"} = $form->parse_amount(\%myconfig, $form->{"qty_$i"});
  $form->{"discount_$i"} = $form->parse_amount(\%myconfig, $form->{"discount_$i"});
  $form->{"sellprice_$i"} = $form->parse_amount(\%myconfig, $form->{"sellprice_$i"});

  $linetotal = sprintf("%.2f", $form->{"sellprice_$i"} * (1 - $form->{"discount_$i"}/100));
  $linetotal = $form->format_amount(\%myconfig, $linetotal * $form->{"qty_$i"});
 
  print qq|
<tr>
  <td><font size=-1><input name="partnumber_$i" size=10 value="$form->{"partnumber_$i"}"></td>
  <td><font size=-1><input name="description_$i" size=30 value="$form->{"description_$i"}"></td>
  <td><font size=-1><input name="qty_$i" size=3 value=|.$form->format_amount(\%myconfig, $form->{"qty_$i"}, "", "nat").qq|></td>
  <td><font size=-1><input name="unit_$i" size=3 value="$form->{"unit_$i"}"></td>
  <td><font size=-1><input name="sellprice_$i" size=7 value=|.$form->format_amount(\%myconfig, $form->{"sellprice_$i"}).qq|></td>
  $taxvalue
  <td><font size=-1><input name="discount_$i" size=3 value=|.$form->format_amount(\%myconfig, $form->{"discount_$i"}, "", "nat").qq|>&nbsp;%</td>
  <td align=right><font size=-1>$linetotal</td>
</tr>

<input type=hidden name="id_$i" value=$form->{"id_$i"}>
<input type=hidden name="inventory_accno_$i" value=$form->{"inventory_accno_$i"}>
<input type=hidden name="income_accno_$i" value=$form->{"income_accno_$i"}>
<input type=hidden name="expense_accno_$i" value=$form->{"expense_accno_$i"}>
<input type=hidden name="listprice_$i" value="$form->{"listprice_$i"}">
<input type=hidden name="assembly_$i" value="$form->{"assembly_$i"}">
<input type=hidden name="taxaccounts_$i" value="$form->{"taxaccounts_$i"}">

<input type=hidden name=rowcount value=$i>

|;

  my $discount = sprintf("%.2f", $form->{"sellprice_$i"} * $form->{"discount_$i"}/100);
  my $linetotal = sprintf("%.2f", $form->{"qty_$i"} * ($form->{"sellprice_$i"} - $discount));

  my $taxtotal = 0;
  my $taxrate = 0;
  
  map { $taxrate += $form->{"${_}_rate"} } split / /, $form->{"taxaccounts_$i"};
  
  if ($form->{taxincluded}) {
    $taxtotal = sprintf("%.2f", $linetotal * (1 - 1 / (1 + $taxrate)));
  } else {
    $taxtotal = sprintf("%.2f", $linetotal * $taxrate);
  }
  
  if ($taxtotal != 0) {
    foreach my $item (split / /, $form->{"taxaccounts_$i"}) {
      $form->{"${item}_total"} += sprintf("%.2f", $taxtotal * $form->{"${item}_rate"}/$taxrate);
      $form->{"${item}_total"} = sprintf("%.2f", $form->{"${item}_total"});
    }
  }

  $form->{invsubtotal} += $linetotal;
  
  if ($form->{taxincluded}) {
    $form->{invtotal} = $form->{invsubtotal};
  } else {
    $form->{invtotal} += $linetotal + $taxtotal;
  }
  
}


sub select_item {
  
  # list items with radio button on a form
  $form->header;

  print qq|

<h2>|.$locale->text('Select from one of the items below').qq|</h2>

<form method=post action=$form->{script}>

<table border=1>
<tr><td>

<table border=0>

<tr bgcolor=336666>
  <th>&nbsp;</th>
  <th><font color=ffffff>|.$locale->text('Number').qq|</th>
  <th><font color=ffffff>|.$locale->text('Description').qq|</th>
  <th><font color=ffffff>|.$locale->text('Price').qq|</th>
|;

  my $i = 0;
  foreach $ref (@{ $form->{item_list} }) {
    $checked = ($i++) ? "" : "checked";
    $ref->{sellprice} = $form->format_amount(\%myconfig, $ref->{sellprice});

    map { $ref->{$_} =~ s/"/&quot;/g } qw(partnumber description unit);
    
    print qq|
<tr>
  <td><input name=ndx class=radio type=radio value=$i $checked></td>
  <td><input name="new_partnumber_$i" type=hidden value="$ref->{partnumber}">$ref->{partnumber}</td>
  <td><input name="new_description_$i" type=hidden value="$ref->{description}">$ref->{description}</td>
  <td align=right><input name="new_sellprice_$i" type=hidden value=$ref->{sellprice}>$ref->{sellprice}</td>
</tr>
<input name="new_listprice_$i" type=hidden value=$ref->{listprice}>
<input name="new_inventory_accno_$i" type=hidden value=$ref->{inventory_accno}>
<input name="new_income_accno_$i" type=hidden value=$ref->{income_accno}>
<input name="new_expense_accno_$i" type=hidden value=$ref->{expense_accno}>
<input name="new_unit_$i" type=hidden value="$ref->{unit}">
<input name="new_assembly_$i" type=hidden value="$ref->{assembly}">
<input name="new_taxaccounts_$i" type=hidden value="$ref->{taxaccounts}">

<input name="new_id_$i" type=hidden value=$ref->{id}>

|;

  }
  
  print qq|
</table>
</td></tr>
</table>
|;

  # delete action variable
  delete $form->{action};
  delete $form->{item_list};
    
  # save all other form variables
  foreach $key (keys %${form}) {
    $form->{$key} =~ s/"/&quot;/g;
    print qq|<input name=$key type=hidden value="$form->{$key}">\n|;
  }

  print qq|
<input type=hidden name=nextsub value=item_selected>

<input class=submit type=submit name=action value="|.$locale->text('Continue').qq|">
</form>|;

}



sub item_selected {

  # replace the last row with the checked row
  $i = $form->{rowcount};

  # index for new item
  $j = $form->{ndx};
  
  map { $form->{"${_}_$i"} = $form->{"new_${_}_$j"} } qw(id partnumber description sellprice listprice inventory_accno income_accno expense_accno unit assembly taxaccounts);

  &display_form;
  
}



sub e_mail_invoice {

  # no email, return
  $form->isblank("email", $locale->text('E-mail address missing!'));
  $form->isblank("invnumber", $locale->text('Invoice Number missing!'));
  $form->isblank("invdate", $locale->text('Invoice Date missing!'));

  &validate_items;

  &customer_details;

  # create the form variables for the invoice
  IS->invoice_details(\%myconfig, \%$form);

  $form->{countrycode} = $myconfig{countrycode};
  $form->{address} =~ s/\\n/<br>/g;
  $form->{signature} =~ s/\\n/<br>/g;

  $form->{invdate} = $locale->date(\%myconfig, $form->{invdate}, 1);
  $form->{duedate} = $locale->date(\%myconfig, $form->{duedate}, 1);

  $form->{version} = "$version";
  $form->{OUT} = "$sendmail";
  
  $form->{templates} = "$myconfig{templates}";
  $form->{IN} = "email_invoice.html";
  $form->{from} = qq|"$myconfig{name}" <$myconfig{email}>|;

  $form->parse_template;


  $form->header;
  
  print qq|
<h2><font color=red>Status</font></h2>

<h4>|.$locale->text('Invoice Number').qq| $form->{invnumber} |.$locale->text('successfully transmitted to').qq| $form->{email}</h4>|;

}



sub print_invoice {

  $form->isblank("invnumber", $locale->text('Invoice Number missing!'));
  $form->isblank("invdate", $locale->text('Invoice Date missing!'));

  &validate_items;

  &customer_details;

  $form->header;

  # create the form variables for the invoice
  IS->invoice_details(\%myconfig, \%$form);

  $form->{address} =~ s/\\n/<br>/g;
  $form->{signature} =~ s/\\n/<br>/g;
  $form->{invdate} = $locale->date(\%myconfig, $form->{invdate}, 1);
  $form->{duedate} = $locale->date(\%myconfig, $form->{duedate}, 1);

  # parse template
  $form->{templates} = "$myconfig{templates}";
  $form->{IN} = "invoice.html";

  $form->parse_template;

}


sub packing_list {

  $form->isblank("invnumber", $locale->text('Invoice Number missing!'));
  $form->isblank("invdate", $locale->text('Invoice Date missing!'));

  &validate_items;

  &customer_details;

  $form->header;

  # create the form variables for the invoice
  IS->invoice_details(\%myconfig, \%$form);

  $form->{address} =~ s/\\n/<br>/g;
  $form->{signature} =~ s/\\n/<br>/g;
  $form->{invdate} = $locale->date(\%myconfig, $form->{invdate}, 1);
  $form->{duedate} = $locale->date(\%myconfig, $form->{duedate}, 1);

  # parse template
  $form->{templates} = "$myconfig{templates}";
  $form->{IN} = "packing_list.html";

  $form->parse_template;

}



sub update_form {


  # if the customer changed get the new values
  ($null, $customer_id) = split /--/, $form->{customer};
  if ($form->{customer_id} ne $customer_id) {
    $form->{customer_id} = $customer_id;
    IS->get_customer(\%myconfig, \%$form);
    $form->{creditlimit} = $form->format_amount(\%myconfig, $form->{creditlimit});
  }

  my $i = $form->{rowcount};

  # if last row empty, check the form otherwise retrieve new item
  if (($form->{"partnumber_$i"} eq "") && ($form->{"description_$i"} eq "")) {

    &check_form;

  } else {

    IS->retrieve_item(\%myconfig, \%$form);
    
    $rows = scalar @{ $form->{item_list} };

    if ($rows) {
      $form->{"qty_$i"}		= ($form->{"qty_$i"} * 1) ? $form->{"qty_$i"} : 1;
      $form->{"discount_$i"}	= $form->format_amount(\%myconfig, $form->{discount} * 100, "", "nat");
      
      if ($rows > 1) {
	&select_item;
	exit;
      } else {
	map { $form->{item_list}[$i]{$_} =~ s/"/&quot;/g } qw(partnumber description unit);
	map { $form->{"${_}_$i"} = $form->{item_list}[0]{$_} } keys %{ $form->{item_list}[0] };
	$form->{"sellprice_$i"} =  $form->format_amount(\%myconfig, $form->{"sellprice_$i"});
	$form->{"listprice_$i"} =  $form->format_amount(\%myconfig, $form->{"listprice_$i"});
	$form->{"qty_$i"} =  $form->format_amount(\%myconfig, $form->{"qty_$i"}, "", "nat");

      }

      &display_form;

    } else {
      $form->error($locale->text("Item not on file!"));
    }
  }
}



sub check_form {
  
  my @a = ();
  my $count = 0;
  my @flds = (qw(id partnumber description qty sellprice unit discount inventory_accno income_accno expense_accno listprice taxaccounts));

  # remove any empty numbers
  for my $i (1 .. $form->{rowcount}) {
    if ($form->{"partnumber_$i"}) {
      push @a, {};
      my $j = $#a;

      map { $a[$j]->{$_} = $form->{"${_}_$i"} } @flds;
      $count++;
    }
  }


  for $i (1 .. $count) {
    my $j = $i - 1;
    map { $form->{"${_}_$i"} = $a[$j]->{$_} } @flds;
  }

  for $i ($count + 1 .. $form->{rowcount}) {
    map { delete $form->{"${_}_$i"} } @flds;
  }

  $form->{rowcount} = $count;

  &display_form;

}



sub post_invoice {

  $form->isblank("invnumber", $locale->text('Invoice Number missing!'));
  $form->isblank("invdate", $locale->text('Invoice Date missing!'));

  &validate_items;

  &customer_details;
  

  ($form->{AR}) = split /--/, $form->{AR};
  ($form->{AR_paid}) = split /--/, $form->{AR_paid};
  

  if (IS->post_invoice(\%myconfig, \%$form)) {
    if ($form->{new_invoice}) {
		    
      $form->header;

      print qq|<form method=post action=$form->{script}>\n|;

      print qq|
<h2><font color=red>|.$locale->text('Confirm!').qq|</font></h2>

<h4>|.$locale->text('Do you want to create another Invoice').qq|</h4>
<p>
<input name=action class=submit type=submit value="|.$locale->text('Yes').qq|">
<input name=action class=submit type=submit value="|.$locale->text('No').qq|">

<input name=callback type=hidden value="$form->{callback}">

<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

</form>
|;

    } else {
      $form->isposted(1);
    }
  }

}


sub yes {

  $form->isposted(1);
  
}


sub no {
  
  $form->{callback} = qq|login.pl?action=company_logo&login=$form->{login}&path=$form->{path}|;

  $form->redirect;
  
}


sub validate_items {
  
  # check if items are valid
  for $i (1 .. $form->{rowcount}) {
    if (!($form->{"income_accno_$i"}) && $form->{"qty_$i"}) {
      $form->isblank("income_accno_$i", $locale->text('Invalid Item in Row') . " $i");
    }
  }

}


sub customer_details {

  unless ($form->{customer_id}) {
    ($null, $customer_id) = split /--/, $form->{customer};
    # get taxes, etc.
    $form->{customer_id} = $customer_id;
    IS->get_customer(\%myconfig, \%$form);
    $form->{creditlimit} = $form->format_amount(\%myconfig, $form->{creditlimit});
  }
  
  ($null, $customer_id) = split /--/, $form->{customer};
  if ($customer_id != $form->{customer_id}) {
    $form->{customer_id} = $customer_id;
    IS->get_customer(\%myconfig, \%$form);
    $form->{creditlimit} = $form->format_amount(\%myconfig, $form->{creditlimit});
  }

  IS->customer_details(\%myconfig, \%$form);
  
  foreach $key (qw(name addr1 addr2 addr3 addr4)) {
    $form->{"shipto$key"} =~ s/"/&quot;/g;
    unless ($form->{"shipto$key"}) {
      $form->{"shipto$key"} = $form->{$key};
    }
  }
				      
}


sub edit {

  $form->{title} = $locale->text('Edit Customer Invoice');
  
  # links for AR, paid and customer
  IS->invoice_links(\%myconfig, \%$form);
  
  &create_links;

  &prepare_invoice;
  
}


sub delete_invoice {

  $form->header;

  print qq|<form method=post action=$form->{script}>\n|;

  foreach $key (keys %$form) {
    $form->{$key} =~ s/"/&quot;/g;
    print qq|<input type=hidden name=$key value="$form->{$key}">\n|;
  }

  print qq|
<h2><font color=red>|.$locale->text('Confirm!').qq|</font></h2>

<h4>|.$locale->text('Are you sure you want to delete Invoice Number').qq| $form->{invnumber}</h4>
<p>
<input name=action class=submit type=submit value="|.$locale->text('Yes Delete Invoice').qq|">
</form>
|;

}



sub yes_delete_invoice {

  $form->isdeleted(IS->delete_invoice(\%myconfig, \%$form));

}



sub stock_assembly {

  $form->{title} = $locale->text('Stock Assembly');
  
  $form->header;
  
  print qq|

<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<table border=1>
<tr valign=top>
<td>
<table>

<tr>
  <th align=right>|.$locale->text('Number').qq|</th>
  <td><input name=partnumber size=20></td>
</tr>

<tr>
  <th align=right>|.$locale->text('Description').qq|</th>
  <td><input name=description size=35></td>
</tr>

</table>
</td>

</tr>
</table>

<p>
<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

<input type=hidden name=nextsub value=list_assemblies>

<input class=submit type=submit name=action value="|.$locale->text('Continue').qq|">
</form>

<p>|.$locale->text('To display all items, leave fields blank');

}



sub list_assemblies {

  IS->retrieve_assemblies(\%myconfig, \%$form);
  

  $column_header{partnumber} = qq|<th><font color=ffffff>|.$locale->text('Number').qq|</th>\n|;
  $column_header{description} = qq|<th><font color=ffffff>|.$locale->text('Description').qq|</th>\n|;
  $column_header{onhand} = qq|<th><font color=ffffff>|.$locale->text('On Hand').qq|</th>\n|;
  $column_header{rop} = qq|<th><font color=ffffff>|.$locale->text('ROP').qq|</th>\n|;
  $column_header{stock} = qq|<th><font color=ffffff>|.$locale->text('Add').qq|</th>\n|;

  @column_index = (qw(partnumber description onhand rop stock));
  
  
  $form->{title} = $locale->text('Stock Assemblies');

  $form->{callback} = "$form->{script}?action=stock_assembly&path=$form->{path}&login=$form->{login}";
  
  $form->header;
  
  print qq|

<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<table border=1>

<tr bgcolor=336666>
|;

  map { print $column_header{$_} } @column_index;

  print qq|</tr>
|;

  $i = 1;
  foreach $ref (@{ $form->{assembly_items} }) {

    map { $ref->{$_} =~ s/"/&quot;/g } qw(partnumber description);
   
    # figure out how many to stock
    $form->{"qty_$i"} = $form->format_amount(\%myconfig, $ref->{rop} - $ref->{onhand}, "", "nat");

    $column_data{partnumber} = qq|<td>$ref->{partnumber}</td>\n|;
    $column_data{description} = qq|<td>$ref->{description}</td>\n|;
    $column_data{onhand} = qq|<td align=right>&nbsp;|.$form->format_amount(\%myconfig, $ref->{onhand}, "", "nat").qq|</td>\n|;
    $column_data{rop} = qq|<td align=right>&nbsp;|.$form->format_amount(\%myconfig, $ref->{rop}, "", "nat").qq|</td>\n|;
    $column_data{stock} = qq|<td><input name="qty_$i" size=6 value=$form->{"qty_$i"}></td>\n|;

    print qq|<tr><input name="id_$i" type=hidden value=$ref->{id}>\n|;
    
    map { print $column_data{$_} } @column_index;
    
    print qq|</tr>|;

    $i++;

  }
  
  $i--;
  print qq|
  
</table>
<input name=rowcount type=hidden value="$i">

<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

<input name=callback type=hidden value="$form->{callback}">

<input type=hidden name=nextsub value=restock_assemblies>

<p>
<input class=submit type=submit name=action value="|.$locale->text('Continue').qq|">

</form>
|;
 
}


sub restock_assemblies {

  IS->restock_assemblies(\%myconfig, \%$form);

  $form->redirect;
  
}


