#!/usr/bin/perl

sub config_main {
  print STDERR "  --> Reading configuration files\n";
  open (FH, "$config{SERVER_ROOT}/conf/dnhttpd.conf")
    or die "     Unable to open configuration file:\n     $!\n";
  my @buffer = <FH>;
  close (FH)
    or die "     Unable to close configuration file:\n     $!\n";
  foreach $line (@buffer) {
    print STDERR "    --> $line";
    chomp $line;
    my ($option, $value) = split(/: /, $line);
    $config{$option} = $value;
  }
  $config{'Document Root'} =~ s/;/:/;
}

sub config_system {
  print STDERR "  --> Determining system parameters\n";

  $name = `uname -s`; chomp ($name);
  $release = `uname -r`; chomp ($release);
  $config{'os'} = "$name $release";
  $config{'hardware'} = `uname -m`; chomp($config{'hardware'});

  print STDERR "    --> Operating system: $config{'os'}\n";
  print STDERR "    --> System type: $config{'hardware'}\n";
}

sub config_vhosts {
  if (!open (FH, "$config{SERVER_ROOT}/conf/vhosts.conf")) {
    print STDERR "  --> No vhosts.conf found, using default document root\n";
  }
  else {
    print STDERR "  --> Reading virtual hosts configuration\n";
    my @buffer = <FH>;
    close FH;
    foreach (@buffer) {
      chomp;
      my ($host, $docroot) = split(/\s/);
      $vhosts{$host} = $docroot;
      print STDERR "    --> $host\n        $docroot\n";
    }
  }
}

sub config_mime {
  if (!open (FH, "$config{SERVER_ROOT}/conf/mime.conf")) {
    print STDERR "  --> Fatal error: mime.conf not found!\n";
  }
  else {
    print STDERR "  --> Reading MIME-type configuration\n";
    my @buffer = <FH>;
    close FH;
    foreach (@buffer) {
      next if ($_ =~ /^\#/);
      chomp;
      my (@columns) = split(/\s+/);
      my $mimetype = shift @columns;
      next unless (@columns);
      foreach $extension (@columns) {
	$mime{$extension} = $mimetype;
	$mime_total++;
      }
    }
    print STDERR "    --> DNHTTPD recognises $mime_total file extensions.\n";
  }
}

sub config_stylesheet {
  print STDERR "  --> Reading server message stylesheet\n";
  if (!($config{'Server message stylesheet'} =~ /^\//)) {
    $config{'Server message stylesheet'} = $config{SERVER_ROOT} . "/" . $config{'Server message stylesheet'};
  }
  if (open (FH, $config{'Server message stylesheet'})) {
    @stylesheet = <FH>;
    close FH;
    print STDERR "    --> Success\n";
  }
  else {
    print STDERR "    --> Failed - stylesheets will not be used\n";
  }
}

sub config_icons {
  if (!open (FH, "$config{SERVER_ROOT}/conf/icons.conf")) {
    print STDERR "  --> Fatal error: icons.conf not found!\n";
  }
  else {
    print STDERR "  --> Reading icons configuration\n";
    my @buffer = <FH>;
    close FH;
    my $count;
    foreach (@buffer) {
      next if ($_=~ /^\#/);
      chomp;
      my (@columns) = split(/\s+/);
      my $iconfilename = shift @columns;
      open (FH, $config{SERVER_ROOT}."/icons/".$iconfilename);
      read (FH, $iconcache{"/icons/".$iconfilename}, (stat($config{SERVER_ROOT}."/icons/".$iconfilename))[7]);
      close FH;
      foreach $extension (@columns) {
	$icons{$extension} = $iconfilename;
      }
      $count++;
    }

    print STDERR "    --> $count icons cached to memory\n";
  }
}

return 1;
