#!/usr/bin/perl

sub load_errors() {
  $error_text[100] = "Continue";
  $error_text[101] = "Switching Protocols";
  $error_text[200] = "OK";
  $error_text[201] = "Created";
  $error_text[202] = "Accepted";
  $error_text[203] = "Non-Authoritative Information";
  $error_text[204] = "No Content";
  $error_text[205] = "Reset Content";
  $error_text[206] = "Partial Content";
  $error_text[300] = "Multiple Choices";
  $error_text[301] = "Moved Permanently";
  $error_text[302] = "Moved Temporarily";
  $error_text[303] = "See Other";
  $error_text[304] = "Not Modified";
  $error_text[305] = "Use Proxy";
  $error_text[400] = "Bad Request";
  $error_text[401] = "Unauthorized";
  $error_text[402] = "Payment Required";
  $error_text[403] = "Forbidden";
  $error_text[404] = "Not Found";
  $error_text[405] = "Method Not Allowed";
  $error_text[406] = "Not Acceptable";
  $error_text[407] = "Proxy Authentication Required";
  $error_text[408] = "Request Time-out";
  $error_text[409] = "Conflict";
  $error_text[410] = "Gone";
  $error_text[411] = "Length Required";
  $error_text[412] = "Precondition Failed";
  $error_text[413] = "Request Entity Too Large";
  $error_text[414] = "Request-URI Too Large";
  $error_text[415] = "Unsupported Media Type";
  $error_text[500] = "Internal Server Error";
  $error_text[501] = "Not Implemented";
  $error_text[502] = "Bad Gateway";
  $error_text[503] = "Service Unavailable";
  $error_text[504] = "Gateway Time-out";
  $error_text[505] = "HTTP Version not supported";
}

sub show_error_page() {
  my ($code, $FH, $text) = @_;

  print $FH "HTTP/1.1 $code $error_text[$code]\r\n";
  print $FH "Server: $config{'Version'}\r\n";
  print $FH "Content-type: text/html\r\n";
  print $FH "\r\n";

  print $FH "<html>\n";
  print $FH "  <head>\n";
  print $FH "    <title>$code $error_text[$code]</title>\n";
  if (@stylesheet) {
    print $FH "    <style type=\"text/css\">\n";
    print $FH "      $_" foreach (@stylesheet);
    print $FH "    </style>\n";
  }
  print $FH "  </head>\n";
  print $FH "  <body>\n";
  print $FH "    <h1>$code $error_text[$code]</h1>\n";
  print $FH "    <p>This error was encountered attempting to access the file you requested.</p>\n";
  if ($text) {
    print $FH "    <p><b>Additional information:</b> $text</p>\n";
  }
  if ($config{Debug} eq "On") {
    print $FH "    <table width=\"100%\" cols=2>\n";
    print $FH "      <tr><td>$_</td><td>$http_request{$_}</td></tr>\n" foreach (keys %http_request);
    print $FH "      <tr><td></td><td></td></tr>\n";
    print $FH "      <tr><td>$_</td><td>$http_parameters{$_}</td></tr>\n" foreach (keys %http_parameters);
    print $FH "    </table>\n";
  }
  print $FH "    <hr>\n";
  print $FH "    <i>$config{'Version'} at ";
  if ($http_parameters{Host}) {
    print $FH $http_parameters{Host};
  }
  else {
    print $FH $config{'HTTP Host'};
  }
  print $FH "</i>\n";
  print $FH "  </body>\n";
  print $FH "</html>\n";

  close $FH;
  exit;
}

sub redirect {
  my $FH = shift;
  my $errorcode = shift;
  my $location = shift;

  print $FH "$http_request{Version} $errorcode $error_text{$errorcode}\r\n";
  print $FH "Server: $config{Version}\r\n";
  print $FH "Location: $location\r\n\r\n";
  close $FH;
  exit;
}

return 1
