#!/usr/bin/perl

sub serve() {
  my $FH = shift;
  binmode $FH;
  my $peeraddress = shift;

  my @buffer;
  my $buffer;
  do {
    $buffer = <$FH>;
    if ($buffer eq "") {
      close $FH;
      print "Error: socket recieved zero length string\n";
      exit;
    }
    push(@buffer, $buffer) unless ($buffer eq "\r\n");
  } until ($buffer eq "\r\n");


  my $first_line = shift @buffer;
  $first_line =~ s/[\r\n]//g;
  my @bits = split(/ /, $first_line);

  # Must be formatted: METHOD OBJECT HTTP-VERSION
  if (@bits != 3) {
    # Error 400 handler
    &show_error_page(400, $FH);
  }

  $http_request{Object} = $bits[1];

  # Check for an unimplemented method
  if (($bits[0] eq "OPTIONS") ||
      ($bits[0] eq "PUT") ||
      ($bits[0] eq "DELETE") ||
      ($bits[0] eq "TRACE"))
  {
    # Error 501 handler
    &show_error_page(501, $FH, "Can't handle the $bits[0] method.");
  }

  # Check for a nonexistent method
  if (($bits[0] ne "GET") and
      ($bits[0] ne "POST") and
      ($bits[0] ne "HEAD"))
  {
    # Error 400 handler
    print LOGGER "INSERT\t$bits[0]\t$http_request{Object}\t\t400\t$peeraddress\tNonexistent method: $bits[0]\n";
    &show_error_page(400, $FH, "HTTP method $bits[0] does not exist");
  }
  else {
    $http_request{Method} = $bits[0];
  }

  # Check that the request object is valid
  if ($http_request{Object} =~ /[:@]/) {
    &show_error_page(400, $FH, "Invalid characters in requested object");
  }

  # Get the query string - if any
  my @object_bits = split(/\?/, $http_request{Object});
  if (@object_bits[2]) {
    &show_error_page(400, $FH, "Query string incorrectly formatted");
  }
  elsif (@object_bits[1]) {
    $http_request{Query} = $object_bits[1];
  }
  $http_request{Path} = $object_bits[0];
  while($http_request{Path} =~ /^(.*)\%(..)(.*)$/) { $http_request{Path} = $1 . chr(hex($2)) . $3; }

  # Check/set the HTTP version
  if (($bits[2] ne "HTTP/1.0") and
      ($bits[2] ne "HTTP/1.1"))
  {
    # Error 505 handler
    &show_error_page(505, $FH, "Version not supported - $bits[2]");
  }
  else {
    $http_request{Version} = $bits[2];
  }

  foreach $line (@buffer) {
    $line =~ s/[\r\n]//g;
    my ($name, $value) = split(/: /, $line);
    $http_parameters{$name} = $value;
  }

  if (($http_request{Path} =~ /\.\./) && ($config{'Parent directories'} eq "No")) {
    &show_error_page(403, $FH, "Specification of the parent directory object \"..\" has been disallowed");
  }

  if (($http_request{Method} eq "GET") ||
      ($http_request{Method} eq "HEAD"))
  {
    &get_head_handler($FH);
  }
  elsif ($http_request{Method} eq "POST") {
    &show_error_page(400, $FH, "Content-length header missing for POST request") unless ($http_parameters{'Content-length'});
    if ((read ($FH, $http_request{Body}, $http_parameters{'Content-length'})) != $http_parameters{'Content-length'}) {
      &show_error_page(400, $FH, "Content-length mismatch");
    }
    &post_handler($FH);
  }


  # The request *should* have been handled by any of the above...
  # if not, something is wrong and a 500 Internal Server Error is shown.
  &show_error_page(500, $FH, "Couldn't find a handler for this request!");
}


return 1;
